@@ -489,7 +489,10 @@ class GenericErrorHelper(ErrorHelper):
                     "generic-search-the-web",
                     "Search the web",
                     "Try performing a web search for\n\n``Python %s: %s``"
-                    % (self.error_info["type_name"], rst_utils.escape(self.error_info["message"])),
+                    % (
+                        self.error_info["type_name"],
+                        rst_utils.escape(self.error_info["message"].replace("\n", " ").strip()),
+                    ),
                     1,
                 )
             )
@@ -804,6 +807,7 @@ class FeedbackDialog(CommonDialog):
             messagebox.showinfo(
                 "Done!",
                 "Thank you for the feedback!\n\nLet us know again when Assistant\nhelps or confuses you!",
+                master=self.master,
             )
             self._close()
         else:
@@ -811,6 +815,7 @@ class FeedbackDialog(CommonDialog):
                 "Problem",
                 "Something went wrong:\n%s\n\nIf you don't mind, then try again later!"
                 % result[:1000],
+                master=self.master,
             )
 
     def _select_unsent_snapshots(self, all_snapshots):
