@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 import ast
-import logging
+from asttokens.asttokens import ASTTokens
 
 
 def extract_text_range(source, text_range):
@@ -31,9 +31,9 @@ def find_expression(node, text_range):
             return node
 
 
-def parse_source(source: bytes, filename="<unknown>", mode="exec"):
+def parse_source(source: bytes, filename="<unknown>", mode="exec", fallback_to_one_char=False):
     root = ast.parse(source, filename, mode)
-    mark_text_ranges(root, source)
+    mark_text_ranges(root, source, fallback_to_one_char)
     return root
 
 
@@ -167,24 +167,12 @@ def get_last_child(node, skip_incorrect=True):
     return None
 
 
-def mark_text_ranges(node, source: bytes):
+def mark_text_ranges(node, source: bytes, fallback_to_one_char=False):
     """
     Node is an AST, source is corresponding source as string.
     Function adds recursively attributes end_lineno and end_col_offset to each node
     which has attributes lineno and col_offset.
     """
-    try:
-        from asttokens import ASTTokens
-    except ImportError:
-        # asttokens may be missing in some configurations
-        from thonny.ast_utils_old_range_marker import old_mark_text_ranges
-
-        logger = logging.getLogger("thonny")
-        if not getattr(logger, "old_astutils_warned", False):
-            logger.warn("asttokens not found, using deprecated alternative")
-            logger.old_astutils_warned = True
-        return old_mark_text_ranges(node, source)
-
     ASTTokens(source.decode("utf8"), tree=node)
     for child in ast.walk(node):
         if hasattr(child, "last_token"):
@@ -193,3 +181,12 @@ def mark_text_ranges(node, source: bytes):
             if hasattr(child, "lineno"):
                 # Fixes problems with some nodes like binop
                 child.lineno, child.col_offset = child.first_token.start
+
+        # some nodes stay without end info
+        if (
+            hasattr(child, "lineno")
+            and (not hasattr(child, "end_lineno") or not hasattr(child, "end_col_offset"))
+            and fallback_to_one_char
+        ):
+            child.end_lineno = child.lineno
+            child.end_col_offset = child.col_offset + 2
