@@ -2,7 +2,6 @@
 import os.path
 import sys
 import tkinter as tk
-import tokenize
 import traceback
 from logging import exception
 from tkinter import messagebox, ttk
@@ -18,9 +17,10 @@ from thonny.common import (
 )
 from thonny.tktextext import rebind_control_a
 from thonny.ui_utils import askopenfilename, asksaveasfilename, select_sequence
-from thonny.misc_utils import running_on_windows
+from thonny.misc_utils import running_on_windows, running_on_mac_os
 from _tkinter import TclError
 from thonny.base_file_browser import choose_node_for_file_operations, ask_backend_path
+import logging
 
 _dialog_filetypes = [("Python files", ".py .pyw"), ("text files", ".txt"), ("all files", ".*")]
 
@@ -34,7 +34,7 @@ class Editor(ttk.Frame):
 
         # parent of codeview will be workbench so that it can be maximized
         self._code_view = CodeView(
-            get_workbench(), propose_remove_line_numbers=True, font="EditorFont"
+            get_workbench(), propose_remove_line_numbers=True, font="EditorFont", replace_tabs=True
         )
         get_workbench().event_generate(
             "EditorTextCreated", editor=self, text_widget=self.get_text_widget()
@@ -47,7 +47,6 @@ class Editor(ttk.Frame):
         self.columnconfigure(0, weight=1)
         self.rowconfigure(0, weight=1)
 
-        self._newlines = None
         self._filename = None
         self._last_known_mtime = None
         self._asking_about_external_change = False
@@ -170,26 +169,22 @@ class Editor(ttk.Frame):
         self.update_appearance()
 
     def _load_local_file(self, filename, keep_undo=False):
-        with tokenize.open(filename) as fp:  # TODO: support also text files
+        with open(filename, "rb") as fp:
             source = fp.read()
 
-        # get the file file format (DOS/UNIX)
-        self._newlines = fp.newlines
-
         # Make sure Windows filenames have proper format
         filename = normpath_with_actual_case(filename)
         self._filename = filename
         self._last_known_mtime = os.path.getmtime(self._filename)
 
         get_workbench().event_generate("Open", editor=self, filename=filename)
-        self._code_view.set_content(source, keep_undo)
+        self._code_view.set_content_as_bytes(source, keep_undo)
         self.get_text_widget().edit_modified(False)
         self._code_view.focus_set()
         self.master.remember_recent_file(filename)
 
     def _load_remote_file(self, filename):
         self._filename = filename
-        self._newlines = None
         self._code_view.set_content("")
         self._code_view.text.set_read_only(True)
 
@@ -207,11 +202,6 @@ class Editor(ttk.Frame):
             raise RuntimeError(response["error"])
 
         content = response["content_bytes"]
-        if content.count(b"\r\n") > content.count(b"\n") / 2:
-            self._newlines = "\r\n"
-        else:
-            self._newlines = "\n"
-
         self._code_view.text.set_read_only(False)
         self._code_view.set_content_as_bytes(content)
         self.get_text_widget().edit_modified(False)
@@ -236,7 +226,7 @@ class Editor(ttk.Frame):
                 "SaveAs", editor=self, filename=save_filename, save_copy=save_copy
             )
 
-        content_bytes = self._code_view.get_content_as_bytes(self._newlines)
+        content_bytes = self._code_view.get_content_as_bytes()
 
         if is_remote_path(save_filename):
             result = self.write_remote_file(save_filename, content_bytes, save_copy)
@@ -280,7 +270,7 @@ class Editor(ttk.Frame):
         return True
 
     def write_remote_file(self, save_filename, content_bytes, save_copy):
-        if get_runner().can_do_file_operations():
+        if get_runner().ready_for_remote_file_operations(show_message=True):
             target_filename = extract_target_path(save_filename)
 
             get_runner().send_command(
@@ -305,7 +295,6 @@ class Editor(ttk.Frame):
             )
             return True
         else:
-            messagebox.showwarning("Can't save", "Device is busy, wait and try again!")
             return False
 
     def ask_new_path(self):
@@ -708,7 +697,7 @@ class EditorNotebook(ui_utils.ClosableNotebook):
 
         if path:
             # self.close_single_untitled_unmodified_editor()
-            self.show_file(path)
+            self.show_file(path, propose_dialog=False)
 
     def _control_o(self, event):
         # http://stackoverflow.com/questions/22907200/remap-default-keybinding-in-tkinter
@@ -818,9 +807,19 @@ class EditorNotebook(ui_utils.ClosableNotebook):
 
         return False
 
-    def show_file(self, filename, text_range=None, set_focus=True):
+    def show_file(self, filename, text_range=None, set_focus=True, propose_dialog=True):
         # self.close_single_untitled_unmodified_editor()
-        editor = self.get_editor(filename, True)
+        try:
+            editor = self.get_editor(filename, True)
+        except PermissionError:
+            logging.getLogger("thonny").exception("Loading " + filename)
+            msg = "Got permission error when trying to load\n" + filename
+            if running_on_mac_os() and propose_dialog:
+                msg += "\n\nTry opening it with File => Open."
+
+            messagebox.showerror("Permission error", msg)
+            return None
+
         assert editor is not None
 
         self.select(editor)
@@ -833,11 +832,7 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         return editor
 
     def show_remote_file(self, target_filename):
-        if not get_runner().can_do_file_operations():
-            messagebox.showwarning(
-                "Can't open",
-                "Device is busy, can't read file content.\n" + "Please try again later!",
-            )
+        if not get_runner().ready_for_remote_file_operations(show_message=True):
             return None
         else:
             return self.show_file(make_remote_path(target_filename))
@@ -858,6 +853,33 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         except TclError:
             pass
 
+        try:
+            self.indicate_modification()
+        except Exception:
+            logging.exception("Could not update modification indication")
+
+    def indicate_modification(self):
+        if not running_on_mac_os():
+            return
+
+        atts = self.winfo_toplevel().wm_attributes()
+        if "-modified" in atts:
+            i = atts.index("-modified")
+            mod = atts[i : i + 2]
+            rest = atts[:i] + atts[i + 2 :]
+        else:
+            mod = ()
+            rest = atts
+
+        for editor in self.get_all_editors():
+            if editor.is_modified():
+                if mod != ("-modified", 1):
+                    self.winfo_toplevel().wm_attributes(*(rest + ("-modified", 1)))
+                break
+        else:
+            if mod == ("-modified", 1):
+                self.winfo_toplevel().wm_attributes(*(rest + ("-modified", 0)))
+
     def _open_file(self, filename):
         editor = Editor(self)
         editor._load_file(filename)
@@ -865,6 +887,8 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         return editor
 
     def get_editor(self, filename, open_when_necessary=False):
+        if not is_remote_path(filename):
+            filename = normpath_with_actual_case(os.path.abspath(filename))
         for child in self.winfo_children():
             child_filename = child.get_filename(False)
             if child_filename == filename:
