@@ -13,6 +13,7 @@ from typing import List, Optional  # @UnusedImport
 import subprocess
 import logging
 import traceback
+from threading import Thread
 
 MESSAGE_MARKER = "\x02"
 
@@ -136,14 +137,6 @@ class InputSubmission(Record):
         self.data = data
 
 
-class InterruptCommand(Record):
-    pass
-
-
-class EOFCommand(Record):
-    pass
-
-
 class CommandToBackend(Record):
     """Command meant for the back-end"""
 
@@ -152,6 +145,20 @@ class CommandToBackend(Record):
         self.name = name
 
 
+class InterruptCommand(CommandToBackend):
+    def __init__(self, **kw) -> None:
+        if "name" in kw:
+            del kw["name"]
+        super().__init__("interrupt", **kw)
+
+
+class EOFCommand(CommandToBackend):
+    def __init__(self, **kw) -> None:
+        if "name" in kw:
+            del kw["name"]
+        super().__init__("eof", **kw)
+
+
 class ToplevelCommand(CommandToBackend):
     def __init__(self, name: str, argv: List[str] = [], **kw) -> None:
         super().__init__(name, **kw)
@@ -220,19 +227,25 @@ def normpath_with_actual_case(name: str) -> str:
     assert os.path.exists(name), "Not exists: " + name
 
     if os.name == "nt":
-        name = os.path.realpath(name)
+        # https://stackoverflow.com/questions/2113822/python-getting-filename-case-as-stored-in-windows/2114975
+        name = os.path.normpath(name)
 
         from ctypes import create_unicode_buffer, windll
 
         buf = create_unicode_buffer(512)
+        # GetLongPathNameW alone doesn't fix filename part
         windll.kernel32.GetShortPathNameW(name, buf, 512)  # @UndefinedVariable
         windll.kernel32.GetLongPathNameW(buf.value, buf, 512)  # @UndefinedVariable
-        if len(buf.value):
-            result = buf.value
-        else:
-            result = name
+        result = buf.value
 
-        assert isinstance(result, str)
+        if result.casefold() != name.casefold():
+            # Sometimes GetShortPathNameW + GetLongPathNameW doesn't work
+            # see eg. https://github.com/thonny/thonny/issues/925
+            windll.kernel32.GetLongPathNameW(name, buf, 512)  # @UndefinedVariable
+            result = buf.value
+
+            if result.casefold() != name.casefold():
+                result = name
 
         if result[1] == ":":
             # ensure drive letter is capital
@@ -240,6 +253,10 @@ def normpath_with_actual_case(name: str) -> str:
         else:
             return result
     else:
+        # easy on Linux
+        # too difficult on mac
+        # https://stackoverflow.com/questions/14515073/in-python-on-osx-with-hfs-how-can-i-get-the-correct-case-of-an-existing-filenam
+        # Hopefully only correct case comes into Thonny (eg. via open dialog)
         return os.path.normpath(name)
 
 
@@ -532,3 +549,56 @@ def get_windows_lnk_target(lnk_file_path):
     result = subprocess.check_output(cmd, universal_newlines=True, timeout=3)
 
     return result.strip()
+
+
+def execute_system_command(cmd, cwd=None, disconnect_stdin=False):
+    env = dict(os.environ).copy()
+    encoding = "utf-8"
+    env["PYTHONIOENCODING"] = encoding
+    # Make sure this python interpreter and its scripts are available
+    # in PATH
+    update_system_path(env, get_augmented_system_path(get_exe_dirs()))
+    popen_kw = dict(
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+        shell=True,
+        env=env,
+        universal_newlines=True,
+        bufsize=0,
+    )
+
+    if cwd and os.path.isdir(cwd):
+        popen_kw["cwd"] = cwd
+
+    if disconnect_stdin:
+        popen_kw["stdin"] = subprocess.DEVNULL
+
+    if sys.version_info >= (3, 6):
+        popen_kw["errors"] = "replace"
+        popen_kw["encoding"] = encoding
+
+    assert cmd.cmd_line.startswith("!")
+    cmd_line = cmd.cmd_line[1:]
+    proc = subprocess.Popen(cmd_line, **popen_kw)
+
+    def copy_stream(source, target):
+        while True:
+            c = source.readline()
+            if c == "":
+                break
+            else:
+                target.write(c)
+                target.flush()
+
+    copy_out = Thread(target=lambda: copy_stream(proc.stdout, sys.stdout), daemon=True)
+    copy_err = Thread(target=lambda: copy_stream(proc.stderr, sys.stderr), daemon=True)
+
+    copy_out.start()
+    copy_err.start()
+    try:
+        proc.wait()
+    except KeyboardInterrupt as e:
+        print(str(e), file=sys.stderr)
+
+    copy_out.join()
+    copy_err.join()
