@@ -9,13 +9,22 @@ import tkinter as tk
 from configparser import ConfigParser
 from logging import exception
 import shutil
-from thonny import THONNY_USER_DIR, get_workbench
+from thonny import THONNY_USER_DIR
 import traceback
 
+_manager_cache = {}
+
 
 def try_load_configuration(filename):
+    if filename in _manager_cache:
+        return _manager_cache[filename]
+
     try:
-        return ConfigurationManager(filename)
+        # use cache so Workbench doesn't create duplicate manager
+        # when FirstRunWindow already created one
+        mgr = ConfigurationManager(filename)
+        _manager_cache[filename] = mgr
+        return mgr
     except configparser.Error:
         from tkinter import messagebox
 
@@ -23,13 +32,10 @@ def try_load_configuration(filename):
             "Problem",
             "Thonny's configuration file can't be read. It may be corrupt.\n\n"
             + "Do you want to discard the file and open Thonny with default settings?",
-            parent=get_workbench()
         ):
             os.replace(filename, filename + "_corrupt")
             # For some reasons Thonny styles are not loaded properly once messagebox has been shown before main window (At least Windows Py 3.5)
-            raise SystemExit(
-                "Configuration file has been discarded. Please restart Thonny!"
-            )
+            raise SystemExit("Configuration file has been discarded. Please restart Thonny!")
         else:
             raise
 
@@ -39,6 +45,7 @@ class ConfigurationManager:
         self._ini = ConfigParser(interpolation=None)
         self._filename = filename
         self._defaults = {}
+        self._defaults_overrides_str = {}
         self._variables = {}  # Tk variables
 
         if os.path.exists(self._filename):
@@ -64,9 +71,7 @@ class ConfigurationManager:
                     # (I know, it's not proper place for this code, but ...)
                     old_user_logs = os.path.join(old_user_dir, "user_logs")
                     new_user_logs = os.path.join(THONNY_USER_DIR, "user_logs")
-                    if os.path.exists(old_user_logs) and not os.path.exists(
-                        new_user_logs
-                    ):
+                    if os.path.exists(old_user_logs) and not os.path.exists(new_user_logs):
                         try:
                             shutil.copytree(old_user_logs, new_user_logs)
                         except Exception:
@@ -74,11 +79,23 @@ class ConfigurationManager:
 
         if not self.get_option("general.configuration_creation_timestamp"):
             self.set_option(
-                "general.configuration_creation_timestamp",
-                datetime.datetime.now().isoformat(),
+                "general.configuration_creation_timestamp", datetime.datetime.now().isoformat()
             )
 
         # print(prefs_filename, self.sections())
+        self._init_default_overrides()
+
+    def _init_default_overrides(self):
+        overrides_path = os.path.join(os.path.dirname(__file__), "defaults.ini")
+        if not os.path.isfile(overrides_path):
+            return
+
+        defparser = configparser.ConfigParser()
+        defparser.read(overrides_path, "utf-8")
+        for section in defparser.sections():
+            for key in defparser[section]:
+                # leave parsing until base default value is known
+                self._defaults_overrides_str[section + "." + key] = defparser[section][key]
 
     def get_option(self, name, secondary_default=None):
         section, option = self._parse_name(name)
@@ -90,12 +107,13 @@ class ConfigurationManager:
 
         try:
             val = self._ini.get(section, option)
-            if name in self._defaults and isinstance(self._defaults[name], str):
-                return val
-            try:
-                return ast.literal_eval(val)
-            except Exception:
+
+            # if option's data type is str (inferred from the default value)
+            # then don't try to parse anything
+            if isinstance(self._defaults.get(name), str):
                 return val
+            else:
+                return self._parse_value(val)
         except Exception:
             if name in self._defaults:
                 return self._defaults[name]
@@ -121,9 +139,21 @@ class ConfigurationManager:
             self._variables[name].set(value)
 
     def set_default(self, name, primary_default_value):
+        # normalize name
         section, option = self._parse_name(name)
         name = section + "." + option
         self._defaults[name] = primary_default_value
+        return
+
+        if name in self._defaults_overrides_str:
+            if isinstance(primary_default_value, str):
+                value = self._defaults_overrides_str[name]
+            else:
+                value = self._parse_value(self._defaults_overrides_str[name])
+        else:
+            value = primary_default_value
+
+        self._defaults[name] = value
 
     def get_variable(self, name: str) -> tk.Variable:
         section, option = self._parse_name(name)
@@ -143,10 +173,7 @@ class ConfigurationManager:
                 var = tk.StringVar(value=value)
             else:
                 raise KeyError(
-                    "Can't create Tk Variable for "
-                    + name
-                    + ". Type is "
-                    + str(type(value))
+                    "Can't create Tk Variable for " + name + ". Type is " + str(type(value))
                 )
             self._variables[name] = var
             return var
@@ -182,3 +209,9 @@ class ConfigurationManager:
             return name.split(".", 1)
         else:
             return "general", name
+
+    def _parse_value(self, value):
+        try:
+            return ast.literal_eval(value)
+        except Exception:
+            return value
