@@ -9,8 +9,6 @@ import sys
 import time
 from typing import Optional, Sequence, Tuple
 
-from thonny import get_workbench
-
 
 def delete_dir_try_hard(path: str, hardness: int = 5) -> None:
     # Deleting the folder on Windows is not so easy task
@@ -39,59 +37,14 @@ def running_on_linux() -> bool:
     return platform.system() == "Linux"
 
 
-def is_hidden_or_system_file(path: str) -> bool:
-    if os.path.basename(path).startswith("."):
-        return True
-    elif running_on_windows():
-        from ctypes import windll
-
-        FILE_ATTRIBUTE_HIDDEN = 0x2
-        FILE_ATTRIBUTE_SYSTEM = 0x4
-        return bool(
-            windll.kernel32.GetFileAttributesW(path)  # @UndefinedVariable
-            & (FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM)
-        )
-    else:
-        return False
+def running_on_rpi() -> bool:
+    return running_on_linux() and (
+        platform.uname().machine.lower().startswith("arm")
+        or os.environ.get("DESKTOP_SESSION") == "LXDE-pi"
+    )
 
 
-def get_win_drives() -> Sequence[str]:
-    # http://stackoverflow.com/a/2288225/261181
-    # http://msdn.microsoft.com/en-us/library/windows/desktop/aa364939%28v=vs.85%29.aspx
-    import string
-    from ctypes import windll
-
-    all_drive_types = [
-        "DRIVE_UNKNOWN",
-        "DRIVE_NO_ROOT_DIR",
-        "DRIVE_REMOVABLE",
-        "DRIVE_FIXED",
-        "DRIVE_REMOTE",
-        "DRIVE_CDROM",
-        "DRIVE_RAMDISK",
-    ]
-
-    required_drive_types = [
-        "DRIVE_REMOVABLE",
-        "DRIVE_FIXED",
-        "DRIVE_REMOTE",
-        "DRIVE_RAMDISK",
-    ]
-
-    drives = []
-    bitmask = windll.kernel32.GetLogicalDrives()  # @UndefinedVariable
-    for letter in string.ascii_uppercase:
-        drive_type = all_drive_types[
-            windll.kernel32.GetDriveTypeW("%s:\\" % letter)
-        ]  # @UndefinedVariable
-        if bitmask & 1 and drive_type in required_drive_types:
-            drives.append(letter + ":\\")
-        bitmask >>= 1
-
-    return drives
-
-
-def list_volumes() -> Sequence[str]:
+def list_volumes(skip_letters=set()) -> Sequence[str]:
     "Adapted from https://github.com/ntoll/uflash/blob/master/uflash.py"
     if os.name == "posix":
         # 'posix' means we're on Linux or OSX (Mac).
@@ -113,6 +66,8 @@ def list_volumes() -> Sequence[str]:
         try:
             volumes = []
             for disk in "ABCDEFGHIJKLMNOPQRSTUVWXYZ":
+                if disk in skip_letters:
+                    continue
                 path = "{}:\\".format(disk)
                 if os.path.exists(path):
                     volumes.append(path)
@@ -136,21 +91,14 @@ def get_win_volume_name(path: str) -> str:
 
     vol_name_buf = ctypes.create_unicode_buffer(1024)
     ctypes.windll.kernel32.GetVolumeInformationW(  # @UndefinedVariable
-        ctypes.c_wchar_p(path),
-        vol_name_buf,
-        ctypes.sizeof(vol_name_buf),
-        None,
-        None,
-        None,
-        None,
-        0,
+        ctypes.c_wchar_p(path), vol_name_buf, ctypes.sizeof(vol_name_buf), None, None, None, None, 0
     )
     assert isinstance(vol_name_buf.value, str)
     return vol_name_buf.value
 
 
-def find_volumes_by_name(volume_name: str) -> Sequence[str]:
-    volumes = list_volumes()
+def find_volumes_by_name(volume_name: str, skip_letters={"A"}) -> Sequence[str]:
+    volumes = list_volumes(skip_letters=skip_letters)
     if os.name == "nt":
         return [
             volume
@@ -162,11 +110,16 @@ def find_volumes_by_name(volume_name: str) -> Sequence[str]:
 
 
 def find_volume_by_name(
-    volume_name: str,
-    not_found_msg: str = "Could not find disk '%s'. Do you want to locate it yourself?",
-    found_several_msg: str = "Found several '%s' disks. Do you want to choose one yourself?",
+    volume_name: str, not_found_msg: Optional[str] = None, found_several_msg: Optional[str] = None
 ) -> Optional[str]:
 
+    # Can't translate in the header as _ may not be available at import time
+    if not_found_msg is None:
+        not_found_msg = _("Could not find disk '%s'. Do you want to locate it yourself?")
+
+    if found_several_msg is None:
+        found_several_msg = _("Found several '%s' disks. Do you want to choose one yourself?")
+
     volumes = find_volumes_by_name(volume_name)
     if len(volumes) == 1:
         return volumes[0]
@@ -179,8 +132,10 @@ def find_volume_by_name(
         from tkinter.messagebox import askyesno
         from thonny.ui_utils import askdirectory
 
-        if askyesno("Can't find suitable disk", msg, parent=get_workbench()):
-            path = askdirectory(master=get_workbench())
+        import tkinter as tk
+
+        if askyesno(_("Can't find suitable disk"), msg):
+            path = askdirectory(master=tk._default_root)
             if path:
                 return path
 
@@ -250,9 +205,7 @@ def _win_get_used_memory():
         if process is None:
             process = get_current_process()
         counters = PROCESS_MEMORY_COUNTERS_EX()
-        ret = GetProcessMemoryInfo(
-            process, ctypes.byref(counters), ctypes.sizeof(counters)
-        )
+        ret = GetProcessMemoryInfo(process, ctypes.byref(counters), ctypes.sizeof(counters))
         if not ret:
             raise ctypes.WinError()
         info = dict((name, getattr(counters, name)) for name, _ in counters._fields_)
@@ -266,8 +219,14 @@ def _unix_get_used_memory():
     "TODO:"
 
 
-def construct_cmd_line(parts):
-    return " ".join(map(shlex.quote, parts))
+def construct_cmd_line(parts, safe_tokens=[]):
+    def quote(s):
+        if s in safe_tokens:
+            return s
+        else:
+            return shlex.quote(s)
+
+    return " ".join(map(quote, parts))
 
 
 def parse_cmd_line(s):
@@ -358,9 +317,7 @@ def levenshtein_damerau_distance(s1, s2, maxDistance):
         for colNum in range(1, l1 + 1):
             insertionCost = curRow[colNum - 1] + 1
             deletionCost = prevRow[colNum] + 1
-            changeCost = prevRow[colNum - 1] + (
-                0 if s1[colNum - 1] == s2[rowNum - 1] else 1
-            )
+            changeCost = prevRow[colNum - 1] + (0 if s1[colNum - 1] == s2[rowNum - 1] else 1)
             #  set the cell value - min distance to reach this
             #  position
             curRow[colNum] = min(insertionCost, deletionCost, changeCost)
@@ -369,13 +326,8 @@ def levenshtein_damerau_distance(s1, s2, maxDistance):
             #  check to see if we have at least 2 characters
             if 1 < rowNum <= colNum:
                 #  test for possible transposition
-                if (
-                    s1[colNum - 1] == s2[colNum - 2]
-                    and s2[colNum - 1] == s1[colNum - 2]
-                ):
-                    curRow[colNum] = min(
-                        curRow[colNum], transpositionRow[colNum - 2] + 1
-                    )
+                if s1[colNum - 1] == s2[colNum - 2] and s2[colNum - 1] == s1[colNum - 2]:
+                    curRow[colNum] = min(curRow[colNum], transpositionRow[colNum - 2] + 1)
 
     #  the last cell of the matrix is ALWAYS the shortest distance between the two strings
     return curRow[-1]
@@ -397,3 +349,126 @@ def get_file_creation_date(path_to_file):
             # We're probably on Linux. No easy way to get creation dates here,
             # so we'll settle for when its content was last modified.
             return stat.st_mtime
+
+
+_timer_time = 0
+
+
+def start_time(text=""):
+    global _timer_time
+    print("RESET", text)
+    _timer_time = time.time()
+
+
+def lap_time(text=""):
+    global _timer_time
+    new_time = time.time()
+    print("LAP", text, round(new_time - _timer_time, 4))
+    _timer_time = time.time()
+
+
+class TimeHelper:
+    def __init__(self, time_allowed):
+        self.start_time = time.time()
+        self.time_allowed = time_allowed
+
+    @property
+    def time_spent(self):
+        return time.time() - self.start_time
+
+    @property
+    def time_left(self):
+        return max(self.time_allowed - self.time_spent, 0)
+
+
+def copy_to_clipboard(data):
+    if running_on_windows():
+        return _copy_to_windows_clipboard(data)
+    elif running_on_mac_os():
+        command = ["pbcopy"]
+    else:
+        command = ["xsel", "-b", "-i"]
+
+    env = dict(os.environ).copy()
+    encoding = "utf-8"
+    env["PYTHONIOENCODING"] = encoding
+
+    if sys.version_info >= (3, 6):
+        extra = {"encoding": encoding}
+    else:
+        extra = {}
+
+    proc = subprocess.Popen(
+        command,
+        stdin=subprocess.PIPE,
+        shell=False,
+        env=env,
+        universal_newlines=True,
+        close_fds=True,
+        **extra
+    )
+    proc.communicate(input=data, timeout=0.1)
+
+
+def _copy_to_windows_clipboard(data):
+    # https://bugs.python.org/file37366/test_clipboard_win.py
+    import ctypes
+
+    wcscpy = ctypes.cdll.msvcrt.wcscpy
+    OpenClipboard = ctypes.windll.user32.OpenClipboard
+    EmptyClipboard = ctypes.windll.user32.EmptyClipboard
+    SetClipboardData = ctypes.windll.user32.SetClipboardData
+    CloseClipboard = ctypes.windll.user32.CloseClipboard
+    CF_UNICODETEXT = 13
+    GlobalAlloc = ctypes.windll.kernel32.GlobalAlloc
+    GlobalLock = ctypes.windll.kernel32.GlobalLock
+    GlobalUnlock = ctypes.windll.kernel32.GlobalUnlock
+    GMEM_DDESHARE = 0x2000
+
+    OpenClipboard(None)
+    EmptyClipboard()
+    hCd = GlobalAlloc(GMEM_DDESHARE, 2 * (len(data) + 1))
+    pchData = GlobalLock(hCd)
+    wcscpy(ctypes.c_wchar_p(pchData), data)
+    GlobalUnlock(hCd)
+    SetClipboardData(CF_UNICODETEXT, hCd)
+    # ctypes.windll.user32.SetClipboardText(CF_UNICODETEXT, hCd)
+    CloseClipboard()
+
+
+def sizeof_fmt(num, suffix="B"):
+    """Readable file size
+    :param num: Bytes value
+    :type num: int
+    :param suffix: Unit suffix (optionnal) default = B
+    :type suffix: str
+    :rtype: str
+    """
+    # https://gist.github.com/cbwar/d2dfbc19b140bd599daccbe0fe925597
+    for unit in ["", "k", "M", "G", "T", "P", "E", "Z"]:
+        if abs(num) < 1024.0:
+            if unit == "":
+                return "%d %s%s" % (num, unit, suffix)
+            return "%.1f %s%s" % (num, unit, suffix)
+        num /= 1024.0
+    return "%.1f%s%s" % (num, "Yi", suffix)
+
+
+def _get_known_folder(ID):
+    # http://stackoverflow.com/a/3859336/261181
+    # http://www.installmate.com/support/im9/using/symbols/functions/csidls.htm
+    import ctypes.wintypes
+
+    SHGFP_TYPE_CURRENT = 0
+    buf = ctypes.create_unicode_buffer(ctypes.wintypes.MAX_PATH)
+    ctypes.windll.shell32.SHGetFolderPathW(0, ID, 0, SHGFP_TYPE_CURRENT, buf)
+    assert buf.value
+    return buf.value
+
+
+def get_roaming_appdata_dir():
+    return _get_known_folder(26)
+
+
+def get_local_appdata_dir():
+    return _get_known_folder(28)
