@@ -20,6 +20,7 @@ class BirdsEyeRunner(Executor):
             source = source.decode("utf-8")
 
         import __main__  # @UnresolvedImport
+
         global_vars = __main__.__dict__
 
         # Following is a trick, which allows importing birdseye in the backends,
@@ -29,8 +30,7 @@ class BirdsEyeRunner(Executor):
 
         eye.exec_string(source, filename, globs=global_vars, locs=global_vars, deep=True)
         port = os.environ.get("BIRDSEYE_PORT", "7777")
-        webbrowser.open_new_tab('http://localhost:%s/ipython_call/' % port
-                                + eye._last_call_id)
+        webbrowser.open_new_tab("http://localhost:%s/ipython_call/" % port + eye._last_call_id)
 
 
 def load_plugin():
