@@ -1,7 +1,7 @@
 from typing import Callable, Dict, Union  # @UnusedImport
 
 from thonny import get_workbench
-from thonny.misc_utils import running_on_linux, running_on_windows
+from thonny.misc_utils import running_on_linux, running_on_windows, running_on_mac_os
 from thonny.workbench import BasicUiThemeSettings, CompoundUiThemeSettings
 
 
@@ -14,12 +14,14 @@ def _treeview_settings() -> BasicUiThemeSettings:
     light_blue = "#ADD8E6"
     light_grey = "#D3D3D3"
 
-    if running_on_linux():
+    if running_on_linux() or running_on_mac_os():
         bg_sel_focus = light_blue
-        fg = "black"
+        fg_sel_focus = "black"
+        fg_sel_notfocus = "black"
     else:
         bg_sel_focus = "SystemHighlight"
-        fg = "SystemHighlightText"
+        fg_sel_focus = "SystemHighlightText"
+        fg_sel_notfocus = "SystemWindowText"
 
     return {
         "Treeview": {
@@ -29,7 +31,10 @@ def _treeview_settings() -> BasicUiThemeSettings:
                     ("selected", "focus", bg_sel_focus),
                     ("selected", "!focus", light_grey),
                 ],
-                "foreground": [("selected", fg)],
+                "foreground": [
+                    ("selected", "focus", fg_sel_focus),
+                    ("selected", "!focus", fg_sel_notfocus),
+                ],
             },
             "layout": [
                 # get rid of borders
@@ -85,7 +90,7 @@ def _text_settings() -> BasicUiThemeSettings:
 
 
 def _label_settings() -> BasicUiThemeSettings:
-    return {"Url.TLabel": {"configure": {"foreground": "#3A66DD"}}}
+    return {"Url.TLabel": {"configure": {"foreground": "DarkBlue"}}}
 
 
 def _button_notebook_settings() -> BasicUiThemeSettings:
@@ -126,10 +131,7 @@ def _button_notebook_settings() -> BasicUiThemeSettings:
                                                 ],
                                             },
                                         ),
-                                        (
-                                            "Notebook.closebutton",
-                                            {"side": "right", "sticky": ""},
-                                        ),
+                                        ("Notebook.closebutton", {"side": "right", "sticky": ""}),
                                     ],
                                 },
                             )
@@ -185,11 +187,7 @@ def clam() -> BasicUiThemeSettings:
                 "relief": "raised",
             },
             "map": {
-                "background": [
-                    ("disabled", frame),
-                    ("pressed", darker),
-                    ("active", lighter),
-                ],
+                "background": [("disabled", frame), ("pressed", darker), ("active", lighter)],
                 "lightcolor": [("pressed", darker)],
                 "darkcolor": [("pressed", darker)],
                 "bordercolor": [("alternate", "#000000")],
@@ -204,11 +202,7 @@ def clam() -> BasicUiThemeSettings:
                     ("pressed", "sunken"),
                     ("active", "raised"),
                 ],
-                "background": [
-                    ("disabled", frame),
-                    ("pressed", darker),
-                    ("active", lighter),
-                ],
+                "background": [("disabled", frame), ("pressed", darker), ("active", lighter)],
                 "lightcolor": [("pressed", darker)],
                 "darkcolor": [("pressed", darker)],
             },
@@ -244,9 +238,7 @@ def clam() -> BasicUiThemeSettings:
                 ]
             },
         },
-        "TMenubutton": {
-            "configure": {"width": scale(11), "padding": scale(5), "relief": "raised"}
-        },
+        "TMenubutton": {"configure": {"width": scale(11), "padding": scale(5), "relief": "raised"}},
         "TEntry": {
             "configure": {"padding": scale(1), "insertwidth": scale(1)},
             "map": {
@@ -263,26 +255,15 @@ def clam() -> BasicUiThemeSettings:
             },
             "map": {
                 "background": [("active", lighter), ("pressed", lighter)],
-                "fieldbackground": [
-                    ("readonly", "focus", selectbg),
-                    ("readonly", frame),
-                ],
+                "fieldbackground": [("readonly", "focus", selectbg), ("readonly", frame)],
                 "foreground": [("readonly", "focus", selectfg)],
                 "arrowcolor": [("disabled", disabledfg)],
             },
         },
-        "ComboboxPopdownFrame": {
-            "configure": {"relief": "solid", "borderwidth": scale(1)}
-        },
+        "ComboboxPopdownFrame": {"configure": {"relief": "solid", "borderwidth": scale(1)}},
         "TSpinbox": {
-            "configure": {
-                "arrowsize": scale(10),
-                "padding": [scale(2), 0, scale(10), 0],
-            },
-            "map": {
-                "background": [("readonly", frame)],
-                "arrowcolor": [("disabled", disabledfg)],
-            },
+            "configure": {"arrowsize": scale(10), "padding": [scale(2), 0, scale(10), 0]},
+            "map": {"background": [("readonly", frame)], "arrowcolor": [("disabled", disabledfg)]},
         },
         "TNotebook.Tab": {
             "configure": {"padding": [scale(6), scale(2), scale(6), scale(2)]},
@@ -315,9 +296,7 @@ def clam() -> BasicUiThemeSettings:
                 "padding": [scale(3), scale(3), scale(3), scale(3)],
             }
         },
-        "TLabelframe": {
-            "configure": {"labeloutside": True, "labelmargins": [0, 0, 0, scale(4)]}
-        },
+        "TLabelframe": {"configure": {"labeloutside": True, "labelmargins": [0, 0, 0, scale(4)]}},
         "TProgressbar": {"configure": {"background": frame}},
         "Sash": {"configure": {"sashthickness": scale(6), "gripcount": 10}},
     }
@@ -337,11 +316,7 @@ def xpnative() -> BasicUiThemeSettings:
             "map": {"foreground": [("disabled", "SystemGrayText")]},
         },
         "TButton": {
-            "configure": {
-                "anchor": "center",
-                "width": scale(11),
-                "padding": [scale(1), scale(1)],
-            }
+            "configure": {"anchor": "center", "width": scale(11), "padding": [scale(1), scale(1)]}
         },
         "Toolbutton": {"configure": {"padding": [scale(4), scale(4)]}},
         "TCheckbutton": {"configure": {"padding": scale(2)}},
@@ -367,9 +342,7 @@ def xpnative() -> BasicUiThemeSettings:
                 "focusfill": [("readonly", "focus", "SystemHighlight")],
             },
         },
-        "ComboboxPopdownFrame": {
-            "configure": {"relief": "solid", "borderwidth": scale(1)}
-        },
+        "ComboboxPopdownFrame": {"configure": {"relief": "solid", "borderwidth": scale(1)}},
         "TSpinbox": {
             "configure": {"padding": [scale(2), 0, scale(14), 0]},
             "map": {
@@ -396,9 +369,7 @@ def xpnative() -> BasicUiThemeSettings:
                 ],
             },
         },
-        "Heading": {  # Treeview heading
-            "configure": {"font": "TkHeadingFont", "relief": "raised"}
-        },
+        "Heading": {"configure": {"font": "TkHeadingFont", "relief": "raised"}},  # Treeview heading
         "TLabelframe.Label": {"configure": {"foreground": "#0046d5"}},
     }
 
@@ -435,11 +406,7 @@ def aqua() -> BasicUiThemeSettings:
         "TButton": {"configure": {"anchor": "center", "width": "6"}},
         "Toolbutton": {"configure": {"padding": 4}},
         "TNotebook": {
-            "configure": {
-                "tabmargins": [10, 0],
-                "tabposition": "n",
-                "padding": [18, 8, 18, 17],
-            }
+            "configure": {"tabmargins": [10, 0], "tabposition": "n", "padding": [18, 8, 18, 17]}
         },
         "TNotebook.Tab": {"configure": {"padding": [12, 3, 12, 2]}},
         "TCombobox": {"configure": {"postoffset": [5, -2, -10, 0]}},
@@ -461,9 +428,7 @@ def aqua() -> BasicUiThemeSettings:
             },
         },
         "TProgressbar": {"configure": {"period": 100, "maxphase": 255}},
-        "Labelframe": {
-            "configure": {"labeloutside": True, "labelmargins": [14, 0, 14, 4]}
-        },
+        "Labelframe": {"configure": {"labeloutside": True, "labelmargins": [14, 0, 14, 4]}},
     }
 
 
@@ -524,6 +489,9 @@ def windows() -> CompoundUiThemeSettings:
             "ViewToolbar.TLabel": {
                 "configure": {"background": "SystemWindow", "padding": [scale(5), 0]}
             },
+            "ViewToolbar.Link.TLabel": {
+                "configure": {"background": "SystemWindow", "padding": [scale(5), 0]}
+            },
             "Active.ViewTab.TLabel": {
                 "configure": {
                     # "font" : "BoldTkDefaultFont",
@@ -531,9 +499,7 @@ def windows() -> CompoundUiThemeSettings:
                     "borderwidth": scale(1),
                 }
             },
-            "Inactive.ViewTab.TLabel": {
-                "configure": {"font": "UnderlineTkDefaultFont"}
-            },
+            "Inactive.ViewTab.TLabel": {"configure": {"font": "UnderlineTkDefaultFont"}},
         },
     ]
 
@@ -590,9 +556,7 @@ def enhanced_aqua() -> CompoundUiThemeSettings:
         # _paned_window_settings(),
         _menu_settings(),
         {
-            "TPanedWindow": {
-                "configure": {"background": "systemDialogBackgroundActive"}
-            },
+            "TPanedWindow": {"configure": {"background": "systemDialogBackgroundActive"}},
             "TFrame": {"configure": {"background": "systemDialogBackgroundActive"}},
             "Tab": {"map": {"foreground": [("selected", "white")]}},
         },
@@ -628,9 +592,3 @@ def load_plugin() -> None:
 
     if "aqua" in original_themes:
         get_workbench().add_ui_theme("Kind of Aqua", "aqua", enhanced_aqua)
-
-    if "Windows" in get_workbench().get_usable_ui_theme_names():
-        get_workbench().set_default("view.ui_theme", "Windows")
-
-    elif "Enhanced Clam" in get_workbench().get_usable_ui_theme_names():
-        get_workbench().set_default("view.ui_theme", "Enhanced Clam")
