@@ -1,7 +1,4 @@
 # -*- coding: utf-8 -*-
-import tkinter as tk
-from tkinter import ttk
-
 from thonny import get_workbench
 from thonny.ui_utils import select_sequence
 
@@ -17,19 +14,10 @@ def load_plugin() -> None:
 
         return handler
 
-    def select_all(event=None):
-        # Tk 8.6 has <<SelectAll>> virtual event, but 8.5 doesn't
-        widget = get_workbench().focus_get()
-        if isinstance(widget, tk.Text):
-            widget.tag_remove("sel", "1.0", "end")
-            widget.tag_add("sel", "1.0", "end")
-        elif isinstance(widget, (ttk.Entry, tk.Entry)):
-            widget.select_range(0, tk.END)
-
     get_workbench().add_command(
         "undo",
         "edit",
-        "Undo",
+        _("Undo"),
         create_edit_command_handler("<<Undo>>"),
         tester=None,  # TODO:
         default_sequence=select_sequence("<Control-z>", "<Command-z>"),
@@ -41,7 +29,7 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "redo",
         "edit",
-        "Redo",
+        _("Redo"),
         create_edit_command_handler("<<Redo>>"),
         tester=None,  # TODO:
         default_sequence=select_sequence("<Control-y>", "<Command-y>"),
@@ -57,7 +45,7 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "Cut",
         "edit",
-        "Cut",
+        _("Cut"),
         create_edit_command_handler("<<Cut>>"),
         tester=None,  # TODO:
         default_sequence=select_sequence("<Control-x>", "<Command-x>"),
@@ -69,7 +57,7 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "Copy",
         "edit",
-        "Copy",
+        _("Copy"),
         create_edit_command_handler("<<Copy>>"),
         tester=None,  # TODO:
         default_sequence=select_sequence("<Control-c>", "<Command-c>"),
@@ -81,7 +69,7 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "Paste",
         "edit",
-        "Paste",
+        _("Paste"),
         create_edit_command_handler("<<Paste>>"),
         tester=None,  # TODO:
         default_sequence=select_sequence("<Control-v>", "<Command-v>"),
@@ -93,8 +81,8 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "SelectAll",
         "edit",
-        "Select all",
-        select_all,
+        _("Select all"),
+        create_edit_command_handler("<<SelectAll>>"),
         tester=None,  # TODO:
         default_sequence=select_sequence("<Control-a>", "<Command-a>"),
         extra_sequences=["<Control-Greek_alpha>"],
