@@ -19,6 +19,7 @@ from thonny.memory import VariablesFrame
 from thonny.misc_utils import shorten_repr, running_on_rpi, running_on_mac_os
 from thonny.tktextext import TextFrame
 from thonny.ui_utils import select_sequence, CommonDialog
+from _tkinter import TclError
 
 _current_debugger = None
 
@@ -41,14 +42,17 @@ class Debugger:
             # tell VM the state we are seeing
             cmd.setdefault(
                 frame_id=self._last_progress_message.stack[-1].id,
-                breakpoints=code.get_current_breakpoints(),
-                cursor_position=self.get_run_to_cursor_breakpoint(),
+                breakpoints=self.get_effective_breakpoints(command),
                 state=self._last_progress_message.stack[-1].event,
                 focus=self._last_progress_message.stack[-1].focus,
                 allow_stepping_into_libraries=get_workbench().get_option(
                     "debugger.allow_stepping_into_libraries"
                 ),
             )
+            if command == "run_to_cursor":
+                # cursor position was added as another breakpoint
+                cmd.name = "resume"
+
             get_runner().send_command(cmd)
             if command == "resume":
                 self.clear_last_frame()
@@ -58,6 +62,18 @@ class Debugger:
     def get_run_to_cursor_breakpoint(self):
         return None
 
+    def get_effective_breakpoints(self, command):
+        result = code.get_current_breakpoints()
+
+        if command == "run_to_cursor":
+            bp = self.get_run_to_cursor_breakpoint()
+            if bp != None:
+                filename, lineno = bp
+                result.setdefault(filename, set())
+                result[filename].add(lineno)
+
+        return result
+
     def command_enabled(self, command):
         if not get_runner().is_waiting_debugger_command():
             return False
@@ -324,7 +340,11 @@ class FrameVisualizer:
         self._frame_id = frame_info.id
         self._filename = frame_info.filename
         self._firstlineno = None
-        self._expression_box = ExpressionBox(text_frame)
+        if running_on_mac_os():
+            self._expression_box = ToplevelExpressionBox(text_frame)
+        else:
+            self._expression_box = PlacedExpressionBox(text_frame)
+
         self._note_box = ui_utils.NoteBox(text_frame.winfo_toplevel())
         self._next_frame_visualizer = None
         self._prev_frame_visualizer = None
@@ -570,9 +590,23 @@ class EditorVisualizer(FrameVisualizer):
         self._decorate_editor_title("")
 
 
-class ExpressionBox(tk.Text):
-    def __init__(self, codeview):
+class BaseExpressionBox:
+    def __init__(self, codeview, text):
+        self.text = text
 
+        self._codeview = codeview
+
+        self._last_focus = None
+        self._last_root_expression = None
+
+        self.text.tag_configure("value", get_syntax_options_for_tag("value"))
+        self.text.tag_configure("before", get_syntax_options_for_tag("active_focus"))
+        self.text.tag_configure("after", get_syntax_options_for_tag("completed_focus"))
+        self.text.tag_configure("exception", get_syntax_options_for_tag("exception_focus"))
+        self.text.tag_raise("exception", "before")
+        self.text.tag_raise("exception", "after")
+
+    def get_text_options(self):
         opts = dict(
             height=1,
             width=1,
@@ -585,21 +619,8 @@ class ExpressionBox(tk.Text):
             wrap=tk.NONE,
             font="EditorFont",
         )
-
         opts.update(get_syntax_options_for_tag("expression_box"))
-
-        tk.Text.__init__(self, codeview.winfo_toplevel(), **opts)
-        self._codeview = codeview
-
-        self._last_focus = None
-        self._last_root_expression = None
-
-        self.tag_configure("value", get_syntax_options_for_tag("value"))
-        self.tag_configure("before", get_syntax_options_for_tag("active_focus"))
-        self.tag_configure("after", get_syntax_options_for_tag("completed_focus"))
-        self.tag_configure("exception", get_syntax_options_for_tag("exception_focus"))
-        self.tag_raise("exception", "before")
-        self.tag_raise("exception", "after")
+        return opts
 
     def update_expression(self, msg, frame_info):
         focus = frame_info.focus
@@ -630,6 +651,7 @@ class ExpressionBox(tk.Text):
                         and msg["exception_info"]["is_fresh"]
                     ),
                 )
+                self._last_focus = focus
 
             self._update_position(frame_info.current_root_expression)
             self._update_size()
@@ -638,7 +660,6 @@ class ExpressionBox(tk.Text):
             # hide and clear on non-expression events
             self.clear_debug_view()
 
-        self._last_focus = focus
         self._last_root_expression = frame_info.current_root_expression
 
     def get_focused_text(self):
@@ -647,29 +668,27 @@ class ExpressionBox(tk.Text):
             end_mark = self._get_mark_name(
                 self._last_focus.end_lineno, self._last_focus.end_col_offset
             )
-            return self.get(start_mark, end_mark)
+            return self.text.get(start_mark, end_mark)
         else:
             return ""
 
     def clear_debug_view(self):
-        if self.winfo_ismapped():
-            self.place_forget()
         self._main_range = None
         self._last_focus = None
         self._clear_expression()
 
     def _clear_expression(self):
-        for tag in self.tag_names():
-            self.tag_remove(tag, "1.0", "end")
+        for tag in self.text.tag_names():
+            self.text.tag_remove(tag, "1.0", "end")
 
-        self.mark_unset(*self.mark_names())
-        self.delete("1.0", "end")
+        self.text.mark_unset(*self.text.mark_names())
+        self.text.delete("1.0", "end")
 
     def _replace(self, focus, value):
         start_mark = self._get_mark_name(focus.lineno, focus.col_offset)
         end_mark = self._get_mark_name(focus.end_lineno, focus.end_col_offset)
 
-        self.delete(start_mark, end_mark)
+        self.text.delete(start_mark, end_mark)
 
         id_str = memory.format_object_id(value.id)
         if get_workbench().in_heap_mode():
@@ -678,12 +697,12 @@ class ExpressionBox(tk.Text):
             value_str = shorten_repr(value.repr, 100)
 
         object_tag = "object_" + str(value.id)
-        self.insert(start_mark, value_str, ("value", object_tag))
+        self.text.insert(start_mark, value_str, ("value", object_tag))
         if misc_utils.running_on_mac_os():
             sequence = "<Command-Button-1>"
         else:
             sequence = "<Control-Button-1>"
-        self.tag_bind(
+        self.text.tag_bind(
             object_tag,
             sequence,
             lambda _: get_workbench().event_generate("ObjectSelect", object_id=value.id),
@@ -699,7 +718,7 @@ class ExpressionBox(tk.Text):
 
         self._clear_expression()
 
-        self.insert("1.0", source)
+        self.text.insert("1.0", source)
 
         # create node marks
         def _create_index(lineno, col_offset):
@@ -712,21 +731,21 @@ class ExpressionBox(tk.Text):
             return str(local_lineno) + "." + str(local_col_offset)
 
         for node in ast.walk(main_node):
-            if "lineno" in node._attributes:
+            if "lineno" in node._attributes and hasattr(node, "end_lineno"):
                 index1 = _create_index(node.lineno, node.col_offset)
                 index2 = _create_index(node.end_lineno, node.end_col_offset)
 
                 start_mark = self._get_mark_name(node.lineno, node.col_offset)
-                if not start_mark in self.mark_names():
-                    self.mark_set(start_mark, index1)
+                if not start_mark in self.text.mark_names():
+                    self.text.mark_set(start_mark, index1)
                     # print("Creating mark", start_mark, index1)
-                    self.mark_gravity(start_mark, tk.LEFT)
+                    self.text.mark_gravity(start_mark, tk.LEFT)
 
                 end_mark = self._get_mark_name(node.end_lineno, node.end_col_offset)
-                if not end_mark in self.mark_names():
-                    self.mark_set(end_mark, index2)
+                if not end_mark in self.text.mark_names():
+                    self.text.mark_set(end_mark, index2)
                     # print("Creating mark", end_mark, index2)
-                    self.mark_gravity(end_mark, tk.RIGHT)
+                    self.text.mark_gravity(end_mark, tk.RIGHT)
 
     def _get_mark_name(self, lineno, col_offset):
         return str(lineno) + "_" + str(col_offset)
@@ -744,9 +763,9 @@ class ExpressionBox(tk.Text):
 
     def _highlight_range(self, text_range, state, has_exception):
         logging.debug("EV._highlight_range: %s", text_range)
-        self.tag_remove("after", "1.0", "end")
-        self.tag_remove("before", "1.0", "end")
-        self.tag_remove("exception", "1.0", "end")
+        self.text.tag_remove("after", "1.0", "end")
+        self.text.tag_remove("before", "1.0", "end")
+        self.text.tag_remove("exception", "1.0", "end")
 
         if state.startswith("after"):
             tag = "after"
@@ -757,10 +776,10 @@ class ExpressionBox(tk.Text):
 
         start_index = self._get_mark_name(text_range.lineno, text_range.col_offset)
         end_index = self._get_mark_name(text_range.end_lineno, text_range.end_col_offset)
-        self.tag_add(tag, start_index, end_index)
+        self.text.tag_add(tag, start_index, end_index)
 
         if has_exception:
-            self.tag_add("exception", start_index, end_index)
+            self.text.tag_add("exception", start_index, end_index)
 
     def _update_position(self, text_range):
         self._codeview.update_idletasks()
@@ -774,25 +793,96 @@ class ExpressionBox(tk.Text):
             x = 30
             y = 30
 
+        self._set_position_make_visible(x, y)
+
+    def _update_size(self):
+        content = self.text.get("1.0", tk.END)
+        lines = content.splitlines()
+        self.text["height"] = len(lines)
+        self.text["width"] = max(map(len, lines))
+
+
+class PlacedExpressionBox(BaseExpressionBox, tk.Text):
+    def __init__(self, codeview):
+        tk.Text.__init__(self, codeview.winfo_toplevel(), self.get_text_options())
+        BaseExpressionBox.__init__(self, codeview, self)
+
+    def clear_debug_view(self):
+        if self.winfo_ismapped():
+            self.place_forget()
+
+        super().clear_debug_view()
+
+    def _set_position_make_visible(self, rel_x, rel_y):
+        x = rel_x
+        y = rel_y
+
         widget = self._codeview.text
-        while widget != self.winfo_toplevel():
+        while widget != self.master:
             x += widget.winfo_x()
             y += widget.winfo_y()
             widget = widget.master
 
-        self.place(x=x, y=y, anchor=tk.NW)
-        self.update_idletasks()
+        if not self.winfo_ismapped():
+            self.place(x=x, y=y, anchor=tk.NW)
+            self.update()
 
-    def _update_size(self):
-        content = self.get("1.0", tk.END)
-        lines = content.splitlines()
-        self["height"] = len(lines)
-        self["width"] = max(map(len, lines))
+
+class ToplevelExpressionBox(BaseExpressionBox, tk.Toplevel):
+    def __init__(self, codeview):
+        tk.Toplevel.__init__(self, codeview.winfo_toplevel())
+        text = tk.Text(self, **self.get_text_options())
+        BaseExpressionBox.__init__(self, codeview, text)
+        self.text.grid()
+
+        if running_on_mac_os():
+            try:
+                # NB! Must be the first thing to do after creation
+                # https://wiki.tcl-lang.org/page/MacWindowStyle
+                self.tk.call(
+                    "::tk::unsupported::MacWindowStyle", "style", self._w, "help", "noActivates"
+                )
+            except TclError:
+                pass
+        else:
+            raise RuntimeError("Should be used only on Mac")
+
+        self.resizable(False, False)
+        self.wm_transient(codeview.winfo_toplevel())
+        self.lift()
+
+    def clear_debug_view(self):
+        if self.winfo_ismapped():
+            self.withdraw()
+
+        super().clear_debug_view()
+
+    def _set_position_make_visible(self, rel_x, rel_y):
+        """
+        widget = self._codeview.text
+        while widget is not None:
+            x += widget.winfo_x()
+            y += widget.winfo_y()
+            widget = widget.master
+        """
+        x = rel_x + self._codeview.text.winfo_rootx()
+        y = rel_y + self._codeview.text.winfo_rooty()
+
+        if not self.winfo_ismapped():
+            self.update()
+            self.deiconify()
+        self.geometry("+%d+%d" % (x, y))
+
+    def get_text_options(self):
+        opts = super().get_text_options()
+        opts["relief"] = "flat"
+        opts["borderwidth"] = 0
+        return opts
 
 
 class DialogVisualizer(CommonDialog, FrameVisualizer):
     def __init__(self, master, frame_info):
-        super().__init__(master)
+        CommonDialog.__init__(self, master)
 
         self.transient(master)
         if misc_utils.running_on_windows():
@@ -823,6 +913,7 @@ class DialogVisualizer(CommonDialog, FrameVisualizer):
 
         self._load_code(frame_info)
         self._text_frame.text.focus()
+        self.update()
 
     def _init_layout_widgets(self, master, frame_info):
         self.main_frame = ttk.Frame(
@@ -1162,7 +1253,11 @@ def load_plugin() -> None:
     global RESUME_COMMAND_CAPTION
     RESUME_COMMAND_CAPTION = _("Resume")
 
-    get_workbench().set_default("debugger.frames_in_separate_windows", True)
+    if get_workbench().in_simple_mode():
+        get_workbench().set_default("debugger.frames_in_separate_windows", False)
+    else:
+        get_workbench().set_default("debugger.frames_in_separate_windows", True)
+
     get_workbench().set_default("debugger.automatic_stack_view", True)
     get_workbench().set_default(
         "debugger.preferred_debugger", "faster" if running_on_rpi() else "nicer"
