@@ -4,7 +4,7 @@ import tkinter as tk
 from tkinter import ttk
 
 from thonny import get_workbench
-from thonny.ui_utils import select_sequence
+from thonny.ui_utils import select_sequence, CommonDialog
 
 # TODO - consider moving the cmd_find method to main class in order to pass the editornotebook reference
 # TODO - logging
@@ -17,7 +17,7 @@ from thonny.ui_utils import select_sequence
 _active_find_dialog = None
 
 
-class FindDialog(tk.Toplevel):
+class FindDialog(CommonDialog):
 
     last_searched_word = None
 
@@ -25,7 +25,7 @@ class FindDialog(tk.Toplevel):
         padx = 15
         pady = 15
 
-        tk.Toplevel.__init__(self, master, takefocus=1, background="pink")
+        super().__init__(master, takefocus=1, background="pink")
         main_frame = ttk.Frame(self)
         main_frame.grid(row=1, column=1, sticky="nsew")
         self.columnconfigure(1, weight=1)
@@ -35,18 +35,14 @@ class FindDialog(tk.Toplevel):
 
         # references to the current set of passive found tags e.g. all words that match the searched term but are not the active string
         self.passive_found_tags = set()
-        self.active_found_tag = (
-            None
-        )  # reference to the currently active (centered) found string
+        self.active_found_tag = None  # reference to the currently active (centered) found string
 
         # a tuple containing the start and indexes of the last processed string
         # if the last action was find, then the end index is start index + 1
         # if the last action was replace, then the indexes correspond to the start
         # and end of the inserted word
         self.last_processed_indexes = None
-        self.last_search_case = (
-            None
-        )  # case sensitivity value used during the last search
+        self.last_search_case = None  # case sensitivity value used during the last search
 
         # set up window display
         self.geometry(
@@ -57,28 +53,24 @@ class FindDialog(tk.Toplevel):
             )
         )
 
-        self.title("Find & Replace")
+        self.title(_("Find & Replace"))
         self.resizable(height=tk.FALSE, width=tk.FALSE)
         self.transient(master)
         self.protocol("WM_DELETE_WINDOW", self._ok)
 
         # Find text label
-        self.find_label = ttk.Label(main_frame, text="Find:")
-        self.find_label.grid(
-            column=0, row=0, sticky="w", padx=(padx, 0), pady=(pady, 0)
-        )
+        self.find_label = ttk.Label(main_frame, text=_("Find:"))
+        self.find_label.grid(column=0, row=0, sticky="w", padx=(padx, 0), pady=(pady, 0))
 
         # Find text field
         self.find_entry_var = tk.StringVar()
         self.find_entry = ttk.Entry(main_frame, textvariable=self.find_entry_var)
-        self.find_entry.grid(
-            column=1, row=0, columnspan=2, padx=(0, 10), pady=(pady, 0)
-        )
+        self.find_entry.grid(column=1, row=0, columnspan=2, padx=(0, 10), pady=(pady, 0))
         if FindDialog.last_searched_word is not None:
             self.find_entry.insert(0, FindDialog.last_searched_word)
 
         # Replace text label
-        self.replace_label = ttk.Label(main_frame, text="Replace with:")
+        self.replace_label = ttk.Label(main_frame, text=_("Replace with:"))
         self.replace_label.grid(column=0, row=1, sticky="w", padx=(padx, 0))
 
         # Replace text field
@@ -96,50 +88,54 @@ class FindDialog(tk.Toplevel):
         # Case checkbox
         self.case_var = tk.IntVar()
         self.case_checkbutton = ttk.Checkbutton(
-            main_frame, text="Case sensitive", variable=self.case_var
+            main_frame, text=_("Case sensitive"), variable=self.case_var
         )
         self.case_checkbutton.grid(column=0, row=3, padx=(padx, 0), pady=(0, pady))
 
         # Direction radiobuttons
         self.direction_var = tk.IntVar()
         self.up_radiobutton = ttk.Radiobutton(
-            main_frame, text="Up", variable=self.direction_var, value=1
+            main_frame, text=_("Up"), variable=self.direction_var, value=1
         )
         self.up_radiobutton.grid(column=1, row=3, pady=(0, pady))
         self.down_radiobutton = ttk.Radiobutton(
-            main_frame, text="Down", variable=self.direction_var, value=2
+            main_frame, text=_("Down"), variable=self.direction_var, value=2
         )
         self.down_radiobutton.grid(column=2, row=3, pady=(0, pady))
         self.down_radiobutton.invoke()
 
         # Find button - goes to the next occurrence
+        button_width = 17
         self.find_button = ttk.Button(
-            main_frame, text="Find", command=self._perform_find, default="active"
-        )
-        self.find_button.grid(
-            column=3, row=0, sticky=tk.W + tk.E, pady=(pady, 0), padx=(0, padx)
+            main_frame,
+            text=_("Find"),
+            width=button_width,
+            command=self._perform_find,
+            default="active",
         )
+        self.find_button.grid(column=3, row=0, sticky=tk.W + tk.E, pady=(pady, 0), padx=(0, padx))
         self.find_button.config(state="disabled")
 
         # Replace button - replaces the current occurrence, if it exists
         self.replace_button = ttk.Button(
-            main_frame, text="Replace", command=self._perform_replace
+            main_frame, text=_("Replace"), width=button_width, command=self._perform_replace
         )
         self.replace_button.grid(column=3, row=1, sticky=tk.W + tk.E, padx=(0, padx))
         self.replace_button.config(state="disabled")
 
         # Replace + find button - replaces the current occurence and goes to next
         self.replace_and_find_button = ttk.Button(
-            main_frame, text="Replace+Find", command=self._perform_replace_and_find
+            main_frame,
+            text=_("Replace+Find"),
+            width=button_width,
+            command=self._perform_replace_and_find,
         )  # TODO - text to resources
-        self.replace_and_find_button.grid(
-            column=3, row=2, sticky=tk.W + tk.E, padx=(0, padx)
-        )
+        self.replace_and_find_button.grid(column=3, row=2, sticky=tk.W + tk.E, padx=(0, padx))
         self.replace_and_find_button.config(state="disabled")
 
         # Replace all button - replaces all occurrences
         self.replace_all_button = ttk.Button(
-            main_frame, text="Replace all", command=self._perform_replace_all
+            main_frame, text=_("Replace all"), width=button_width, command=self._perform_replace_all
         )  # TODO - text to resources
         self.replace_all_button.grid(
             column=3, row=3, sticky=tk.W + tk.E, padx=(0, padx), pady=(0, pady)
@@ -233,7 +229,7 @@ class FindDialog(tk.Toplevel):
 
         tofind = self.find_entry.get()
         if len(tofind) == 0:
-            self.infotext_label_var.set("Enter string to be replaced.")
+            self.infotext_label_var.set(_("Enter string to be replaced."))
             return
 
         toreplace = self.replace_entry.get()
@@ -257,9 +253,7 @@ class FindDialog(tk.Toplevel):
             if toreplace != "":
                 self.codeview.text.insert(currentpos, toreplace)
 
-            currentpos = self.codeview.text.index(
-                "%s+%dc" % (currentpos, len(toreplace))
-            )
+            currentpos = self.codeview.text.index("%s+%dc" % (currentpos, len(toreplace)))
 
         get_workbench().event_generate(
             "ReplaceAll", widget=self.codeview.text, old_text=tofind, new_text=toreplace
@@ -306,9 +300,7 @@ class FindDialog(tk.Toplevel):
             search_start_index = self.codeview.text.index(
                 "insert"
             )  # start searching from the current insert position
-            self._find_and_tag_all(
-                tofind
-            )  # set the passive tag to ALL found occurences
+            self._find_and_tag_all(tofind)  # set the passive tag to ALL found occurences
             FindDialog.last_searched_word = tofind  # set the data about last search
             self.last_search_case = self._is_search_case_sensitive()
 
@@ -321,7 +313,7 @@ class FindDialog(tk.Toplevel):
         )  # performs the search and sets the start index of the found string
         if len(wordstart) == 0:
             self.infotext_label_var.set(
-                "The specified text was not found!"
+                _("The specified text was not found!")
             )  # TODO - better text, also move it to the texts resources list
             self.replace_and_find_button.config(state="disabled")
             self.replace_button.config(state="disabled")
@@ -361,9 +353,7 @@ class FindDialog(tk.Toplevel):
     # removes the active tag and all passive tags
     def _remove_all_tags(self):
         for tag in self.passive_found_tags:
-            self.codeview.text.tag_remove(
-                "found", tag[0], tag[1]
-            )  # removes the passive tags
+            self.codeview.text.tag_remove("found", tag[0], tag[1])  # removes the passive tags
 
         if self.active_found_tag is not None:
             self.codeview.text.tag_remove(
@@ -419,7 +409,7 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "OpenFindDialog",
         "edit",
-        "Find & Replace",
+        _("Find & Replace"),
         cmd_open_find_dialog,
         default_sequence=select_sequence("<Control-f>", "<Command-f>"),
         extra_sequences=["<Control-Greek_phi>"],
