@@ -11,6 +11,7 @@ from thonny.common import (
     serialize_message,
     BackendEvent,
     ValueInfo,
+    execute_system_command,
 )
 import sys
 import logging
@@ -52,8 +53,8 @@ INTERRUPT_CMD = b"\x03"
 SOFT_REBOOT_CMD = b"\x04"
 
 # Output tokens
-THONNY_MSG_START = b"\x02"
-THONNY_MSG_END = b"\x04"
+THONNY_MSG_START = b"\x02<thonny>"
+THONNY_MSG_END = b"</thonny>\x04"
 EOT = b"\x04"
 NORMAL_PROMPT = b">>> "
 LF = b"\n"
@@ -63,6 +64,7 @@ OK = b"OK"
 # Looks like it's not translatable in CP
 # https://github.com/adafruit/circuitpython/blob/master/locale/circuitpython.pot
 FIRST_RAW_PROMPT = b"raw REPL; CTRL-B to exit\r\n>"
+FIRST_RAW_PROMPT_SUFFIX = b"\r\n>"
 
 RAW_PROMPT = b">"
 
@@ -82,6 +84,7 @@ def debug(msg):
 class MicroPythonBackend:
     def __init__(self, connection, clean, api_stubs_path):
         self._connection = connection
+        self._local_cwd = None
         self._cwd = None
         self._interrupt_requested = False
         self._cancel_requested = False
@@ -125,7 +128,14 @@ class MicroPythonBackend:
                 self._cancel_requested = False
                 self._interrupt_requested = False
                 self._check_for_connection_errors()
-                cmd = self._command_queue.get(timeout=0.1)
+                try:
+                    cmd = self._command_queue.get(timeout=0.1)
+                except Empty:
+                    # No command in queue, but maybe a thread produced output meanwhile
+                    # or the user resetted the device
+                    self._forward_unexpected_output()
+                    continue
+
                 if isinstance(cmd, InputSubmission):
                     self._submit_input(cmd.data)
                 elif isinstance(cmd, EOFCommand):
@@ -134,8 +144,6 @@ class MicroPythonBackend:
                     self._interrupt()
                 else:
                     self.handle_command(cmd)
-            except Empty:
-                self._check_for_idle_events()
             except KeyboardInterrupt:
                 self._interrupt()
 
@@ -149,7 +157,7 @@ class MicroPythonBackend:
         self._connection.write(RAW_MODE_CMD)
         self._connection.read_until(FIRST_RAW_PROMPT)
 
-        return welcome_text.decode(ENCODING)
+        return welcome_text.decode(ENCODING, errors="replace")
 
     def _fetch_uname(self):
         res = self._evaluate("__thonny_os.uname()", prelude="import os as __thonny_os")
@@ -244,8 +252,12 @@ class MicroPythonBackend:
 
         discarded_bytes = b""
 
-        for delay in [0.05, 0.5, 0.1, 2.0]:
+        for delay in [0.05, 0.5, 0.1, 1.0, 3.0, 5.0]:
             # Interrupt several times, because with some drivers first interrupts seem to vanish
+            if delay >= 1:
+                self._show_error(
+                    "Could not enter REPL. Trying again with %d second waiting time..." % delay
+                )
             self._connection.reset_output_buffer()
             self._connection.write(INTERRUPT_CMD)
             self._connection.write(RAW_MODE_CMD)
@@ -254,7 +266,25 @@ class MicroPythonBackend:
             if discarded_bytes.endswith(FIRST_RAW_PROMPT) or discarded_bytes.endswith(b"\r\n>"):
                 break
         else:
-            raise TimeoutError("Can't get to raw prompt. Read bytes: " + str(discarded_bytes))
+            max_tail_length = 500
+            if len(discarded_bytes) > max_tail_length:
+                discarded_bytes_str = (
+                    "[skipping %d bytes] ..." % (len(discarded_bytes) - max_tail_length)
+                ) + repr(discarded_bytes[:-max_tail_length])
+            else:
+                discarded_bytes_str = repr(discarded_bytes)
+            self._show_error(
+                "Could not enter REPL. Giving up. Read bytes:\n"
+                + discarded_bytes_str
+                + "\n\nYour options:\n\n"
+                + "  - check connection properties;\n"
+                + "  - make sure the device has suitable firmware;\n"
+                + "  - make sure the device is not in bootloader mode;\n"
+                + "  - reset the device and try again;\n"
+                + "  - try other serial clients (Putty, TeraTerm, screen, ...);\n"
+                + "  - ask for help in Thonny's forum or issue tracker."
+            )
+            sys.exit()
 
     def _soft_reboot(self, side_command):
         if side_command:
@@ -289,7 +319,13 @@ class MicroPythonBackend:
     def handle_command(self, cmd):
         assert isinstance(cmd, (ToplevelCommand, InlineCommand))
 
+        if "local_cwd" in cmd:
+            self._local_cwd = cmd["local_cwd"]
+
         def create_error_response(**kw):
+            if not "error" in kw:
+                kw["error"] = traceback.format_exc()
+
             if isinstance(cmd, ToplevelCommand):
                 return ToplevelResponse(command_name=cmd.name, **kw)
             else:
@@ -383,8 +419,16 @@ class MicroPythonBackend:
             "Press Ctrl-C to enter the REPL. Use CTRL-D to reload.",
         )
 
+    def _ensure_raw_propmt(self):
+        # similar to _interrupt_to_raw_prompt, but assumes we are already in a prompt
+        self._forward_unexpected_output()
+        self._connection.write(RAW_MODE_CMD)
+        prompt = self._connection.read_until(FIRST_RAW_PROMPT_SUFFIX, 1, True)
+        if not prompt.endswith(FIRST_RAW_PROMPT_SUFFIX):
+            raise TimeoutError("Could not ensure raw prompt")
+
     def _execute(self, script, capture_output=False):
-        # self._ensure_raw_propmt()
+        self._ensure_raw_propmt()
 
         # send command
         self._connection.write(script.encode(ENCODING) + EOT)
@@ -474,15 +518,12 @@ class MicroPythonBackend:
                 and not self._ctrl_suggestion_given
                 and time.time() - self._startup_time > 1.5
             ):
-                self._send_output(
+                self._show_error(
                     "\n"
                     + "Device is busy or does not respond. Your options:\n\n"
-                    + "  - check the connection properties;\n"
-                    + "  - make sure the device has suitable firmware;\n"
-                    + "  - make sure the device is not in bootloader mode;\n"
-                    + "  - wait until current work is complete;\n"
-                    + "  - use Ctrl+C to interrupt current work.\n",
-                    "stderr",
+                    + "  - wait until it completes current work;\n"
+                    + "  - use Ctrl+C to interrupt current work;\n"
+                    + "  - use Stop/Restart to interrupt more and enter REPL.\n"
                 )
                 self._ctrl_suggestion_given = True
 
@@ -494,7 +535,8 @@ class MicroPythonBackend:
             # Process input in chunks (max 1 parsing marker per chunk).
             # Prefer whole lines (to reduce the number of events),
             # but don't wait too long for eol.
-            output += self._connection.soft_read_until(BLOCK_CLOSERS, timeout=0.05)
+            ddd = self._connection.soft_read_until(BLOCK_CLOSERS, timeout=0.05)
+            output += ddd
             stream_name = "stderr" if eot_count == 1 else "stdout"
 
             if output.endswith(THONNY_MSG_START):
@@ -550,7 +592,7 @@ class MicroPythonBackend:
             if output.endswith(FIRST_RAW_PROMPT[:-1]):
                 # incomplete raw prompt, wait for more
                 pass
-            else:
+            elif output:
                 if capture_output:
                     if stream_name == "stdout":
                         out += output
@@ -558,14 +600,13 @@ class MicroPythonBackend:
                         assert stream_name == "stderr"
                         err += output
                 else:
-                    # TODO: deal with partial UTF-8 chars
-                    self._send_output(output.decode(ENCODING), stream_name)
+                    self._send_output(output.decode(ENCODING, errors="replace"), stream_name)
                 output = b""
 
         debug("doneproc")
         return (
-            out.decode(ENCODING),
-            err.decode(ENCODING),
+            out.decode(ENCODING, errors="replace"),
+            err.decode(ENCODING, errors="replace"),
             None if value is None else value.decode(ENCODING),
         )
 
@@ -601,8 +642,23 @@ class MicroPythonBackend:
         while postponed:
             self._command_queue.put(postponed.pop(0))
 
-    def _check_for_idle_events(self):
-        self._send_output(self._connection.read_all().decode(ENCODING, "replace"), "stdout")
+    def _forward_unexpected_output(self):
+        "Invoked between commands"
+        data = self._connection.read_all()
+        if data.endswith(NORMAL_PROMPT):
+            # looks like the device was resetted
+
+            # hide the regular prompt from the output ...
+            data = data[: -len(NORMAL_PROMPT)]
+            at_prompt = True
+        else:
+            at_prompt = False
+
+        self._send_output(data.decode(ENCODING, "replace"), "stdout")
+        if at_prompt:
+            # ... and recreate Thonny prompt
+            self.send_message(ToplevelResponse())
+
         self._check_for_connection_errors()
 
     def _supports_directories(self):
@@ -647,6 +703,10 @@ class MicroPythonBackend:
             self._execute(cmd.source)
             return {}
 
+    def _cmd_execute_system_command(self, cmd):
+        # Can't use stdin, because a thread is draining it
+        execute_system_command(cmd, cwd=self._local_cwd, disconnect_stdin=True)
+
     def _cmd_get_globals(self, cmd):
         if cmd.module_name == "__main__":
             globs = self._evaluate(
@@ -820,6 +880,7 @@ class MicroPythonBackend:
             completions = script.completions()
             result["completions"] = self._filter_completions(completions)
         except Exception:
+            traceback.print_exc()
             result["error"] = "Autocomplete error"
 
         return result
@@ -831,18 +892,19 @@ class MicroPythonBackend:
             if completion.name.startswith("__"):
                 continue
 
-            parent_name = completion.parent().name
-            name = completion.name
-            root = completion.full_name.split(".")[0]
+            if completion.parent() and completion.full_name:
+                parent_name = completion.parent().name
+                name = completion.name
+                root = completion.full_name.split(".")[0]
 
-            # jedi proposes names from CPython builtins
-            if root in self._builtins_info and name not in self._builtins_info[root]:
-                continue
+                # jedi proposes names from CPython builtins
+                if root in self._builtins_info and name not in self._builtins_info[root]:
+                    continue
 
-            if parent_name == "builtins" and name not in self._builtins_info:
-                continue
+                if parent_name == "builtins" and name not in self._builtins_info:
+                    continue
 
-            result.append({"name": name, "complete": completion.complete})
+            result.append({"name": completion.name, "complete": completion.complete})
 
         return result
 
@@ -876,7 +938,11 @@ class MicroPythonBackend:
                 prefix = match.group()
                 if "." in prefix:
                     obj, prefix = prefix.rsplit(".", 1)
-                    names = self._evaluate("dir(%s)" % obj)
+                    names = self._evaluate(
+                        "dir({}) if '{}' in locals() or '{}' in globals() else []".format(
+                            obj, obj, obj
+                        )
+                    )
                 else:
                     names = self._evaluate("dir()")
             else:
@@ -884,6 +950,10 @@ class MicroPythonBackend:
                 prefix = ""
 
             completions = []
+
+            # prevent TypeError (iterating over None)
+            names = names if names else []
+
             for name in names:
                 if name.startswith(prefix) and not name.startswith("__"):
                     completions.append({"name": name, "complete": name[len(prefix) :]})
@@ -1071,6 +1141,7 @@ class MicroPythonBackend:
                     def __W(x):
                         global __thonny_written
                         __thonny_written += __thonny_fp.write(__thonny_unhex(x))
+                        __thonny_fp.flush()
                 """
                     )
                 )
@@ -1571,13 +1642,20 @@ class MicroPythonBackend:
                             if __thonny_name.startswith('.') or __thonny_name == "System Volume Information":
                                 continue
                             __thonny_full = (__thonny_real_path + '/' + __thonny_name).replace("//", "/")
-                            __thonny_st = __thonny_os.stat(__thonny_full)
-                            if __thonny_st[0] & 0o170000 == 0o040000:
-                                # directory
-                                __thonny_children[__thonny_name] = {"kind" : "dir", "size" : None}
-                            else:
-                                __thonny_children[__thonny_name] = {"kind" : "file", "size" :__thonny_st[6]}
-                            __thonny_children[__thonny_name]["time"] = max(__thonny_st[8], __thonny_st[9])  
+                            try:
+                                __thonny_st = __thonny_os.stat(__thonny_full)
+                                if __thonny_st[0] & 0o170000 == 0o040000:
+                                    # directory
+                                    __thonny_children[__thonny_name] = {"kind" : "dir", "size" : None}
+                                else:
+                                    __thonny_children[__thonny_name] = {"kind" : "file", "size" :__thonny_st[6]}
+                                
+                                # converting from 2000-01-01 epoch to Unix epoch 
+                                __thonny_children[__thonny_name]["time"] = max(__thonny_st[8], __thonny_st[9]) + 946684800
+                            except OverflowError:
+                                # Probably "System Volume Information" in trinket
+                                # https://github.com/thonny/thonny/issues/923
+                                pass
                             
                     __thonny_result[__thonny_path] = __thonny_children                            
             """
@@ -1605,8 +1683,12 @@ class MicroPythonBackend:
         if error:
             message += " (" + str(error) + ")"
         self._send_output("\n" + message + "\n", "stderr")
+        self._send_output("\n" + "Use Stop/Restart to reconnect." + "\n", "stderr")
         sys.exit(EXPECTED_TERMINATION_CODE)
 
+    def _show_error(self, msg):
+        self._send_output(msg + "\n", "stderr")
+
 
 class ExecutionError(Exception):
     pass
