@@ -31,9 +31,7 @@ class ObjectInspector(ttk.Frame):
         self.rowconfigure(1, weight=1)
 
         get_workbench().bind("ObjectSelect", self.show_object, True)
-        get_workbench().bind(
-            "get_object_info_response", self._handle_object_info_event, True
-        )
+        get_workbench().bind("get_object_info_response", self._handle_object_info_event, True)
         get_workbench().bind("DebuggerResponse", self._handle_progress_event, True)
         get_workbench().bind("ToplevelResponse", self._handle_progress_event, True)
 
@@ -76,7 +74,7 @@ class ObjectInspector(ttk.Frame):
                     page.grid(row=1, column=0, sticky="nsew", padx=0)
                     tab.configure(style="Active.ViewTab.TLabel")
                     if self.active_page == self.attributes_page and (
-                        self.object_info is None or self.object_info["attributes"] == {}
+                        self.object_info is None or not self.object_info.get("attributes")
                     ):
                         self.request_object_info()
 
@@ -107,10 +105,10 @@ class ObjectInspector(ttk.Frame):
         self.title_label.bind("<Configure>", configure, True)
 
         self.back_button = create_navigation_link(
-            1, "nav-backward", self.navigate_back, "Previous object", (5, 0)
+            1, "nav-backward", self.navigate_back, _("Previous object"), (5, 0)
         )
         self.forward_button = create_navigation_link(
-            2, "nav-forward", self.navigate_forward, "Next object"
+            2, "nav-forward", self.navigate_forward, _("Next object")
         )
         self.back_links = []
         self.forward_links = []
@@ -249,7 +247,7 @@ class ObjectInspector(ttk.Frame):
             self.back_label.config(foreground="lightgray", cursor="arrow")
         else:
             self.back_label.config(foreground="blue", cursor="hand2")
-    
+
         if self.forward_links == []:
             self.forward_label.config(foreground="lightgray", cursor="arrow")
         else:
@@ -279,9 +277,7 @@ class ObjectInspector(ttk.Frame):
 
     def goto_type(self, event):
         if self.object_info is not None:
-            get_workbench().event_generate(
-                "ObjectSelect", object_id=self.object_info["type_id"]
-            )
+            get_workbench().event_generate("ObjectSelect", object_id=self.object_info["type_id"])
 
 
 class ContentInspector:
@@ -340,10 +336,10 @@ class FileHandleInspector(TextFrame, ContentInspector):
 
         # TODO: show this info somewhere
         """
-        label.configure(text="Read %d/%d %s, %d/%d %s" 
+        label.configure(text="Read %d/%d %s, %d/%d %s"
                         % (read_char_count,
                            char_count,
-                           "symbol" if char_count == 1 else "symbols",  
+                           "symbol" if char_count == 1 else "symbols",
                            read_line_count_term,
                            line_count_term,
                            "line" if line_count_term == 1 else "lines"))
@@ -386,10 +382,10 @@ class StringInspector(TextFrame, ContentInspector):
         self.text.configure(height=min(line_count_sep, 10))
         self.text.set_content(content)
         """ TODO:
-        label.configure(text="%d %s, %d %s" 
+        label.configure(text="%d %s, %d %s"
                         % (len(content),
                            "symbol" if len(content) == 1 else "symbols",
-                           line_count_term, 
+                           line_count_term,
                            "line" if line_count_term == 1 else "lines"))
         """
 
@@ -412,10 +408,10 @@ class ReprInspector(TextFrame, ContentInspector):
         line_count_sep = len(content.split("\n"))
         line_count_term = len(content.splitlines())
         self.text.configure(height=min(line_count_sep, 10))
-        label.configure(text="%d %s, %d %s" 
+        label.configure(text="%d %s, %d %s"
                         % (len(content),
                            "symbol" if len(content) == 1 else "symbols",
-                           line_count_term, 
+                           line_count_term,
                            "line" if line_count_term == 1 else "lines"))
         """
 
@@ -482,9 +478,7 @@ class ElementsInspector(thonny.memory.MemoryFrame, ContentInspector):
 
             self.tree.set(node_id, "id", thonny.memory.format_object_id(element.id))
             self.tree.set(
-                node_id,
-                "value",
-                shorten_repr(element.repr, thonny.memory.MAX_REPR_LENGTH_IN_GRID),
+                node_id, "value", shorten_repr(element.repr, thonny.memory.MAX_REPR_LENGTH_IN_GRID)
             )
             index += 1
 
@@ -494,16 +488,14 @@ class ElementsInspector(thonny.memory.MemoryFrame, ContentInspector):
         """ TODO:
         label.configure (
             text=("%d element" if count == 1 else "%d elements") % count
-        ) 
+        )
         """
 
 
 class DictInspector(thonny.memory.MemoryFrame, ContentInspector):
     def __init__(self, master):
         ContentInspector.__init__(self, master)
-        thonny.memory.MemoryFrame.__init__(
-            self, master, ("key_id", "id", "key", "value")
-        )
+        thonny.memory.MemoryFrame.__init__(self, master, ("key_id", "id", "key", "value"))
         self.configure(border=1)
         # self.vert_scrollbar.grid_remove()
         self.tree.column("key_id", width=100, anchor=tk.W, stretch=False)
@@ -543,15 +535,11 @@ class DictInspector(thonny.memory.MemoryFrame, ContentInspector):
             node_id = self.tree.insert("", "end")
             self.tree.set(node_id, "key_id", thonny.memory.format_object_id(key.id))
             self.tree.set(
-                node_id,
-                "key",
-                shorten_repr(key.repr, thonny.memory.MAX_REPR_LENGTH_IN_GRID),
+                node_id, "key", shorten_repr(key.repr, thonny.memory.MAX_REPR_LENGTH_IN_GRID)
             )
             self.tree.set(node_id, "id", thonny.memory.format_object_id(value.id))
             self.tree.set(
-                node_id,
-                "value",
-                shorten_repr(value.repr, thonny.memory.MAX_REPR_LENGTH_IN_GRID),
+                node_id, "value", shorten_repr(value.repr, thonny.memory.MAX_REPR_LENGTH_IN_GRID)
             )
 
         count = len(object_info["entries"])
@@ -560,7 +548,7 @@ class DictInspector(thonny.memory.MemoryFrame, ContentInspector):
         """ TODO:
         label.configure (
             text=("%d entry" if count == 1 else "%d entries") % count
-        ) 
+        )
         """
 
         self.update_memory_model()
@@ -583,8 +571,7 @@ class ImageInspector(ContentInspector, tk.Frame):
             data = object_info["image_data"]
         else:
             self.label.configure(
-                image=None,
-                text="Unsupported image data (%s)" % type(object_info["image_data"]),
+                image=None, text="Unsupported image data (%s)" % type(object_info["image_data"])
             )
             return
 
@@ -611,4 +598,4 @@ class AttributesFrame(thonny.memory.VariablesFrame):
 
 
 def load_plugin() -> None:
-    get_workbench().add_view(ObjectInspector, "Object inspector", "se")
+    get_workbench().add_view(ObjectInspector, _("Object inspector"), "se")
