@@ -42,6 +42,7 @@ from thonny.common import (
     path_startswith,
     serialize_message,
     update_system_path,
+    EOFCommand,
 )
 from thonny.misc_utils import construct_cmd_line, running_on_mac_os, running_on_windows
 
@@ -49,6 +50,9 @@ from typing import Any, List, Optional, Sequence, Set  # @UnusedImport; @UnusedI
 from thonny.terminal import run_in_terminal
 from thonny.ui_utils import select_sequence, show_dialog, CommonDialogEx
 from tkinter import messagebox
+import warnings
+import re
+import traceback
 
 
 WINDOWS_EXE = "python.exe"
@@ -58,11 +62,15 @@ RUN_COMMAND_LABEL = ""  # init later when gettext is ready
 RUN_COMMAND_CAPTION = ""
 EDITOR_CONTENT_TOKEN = "$EDITOR_CONTENT"
 
-EXPECTED_TERMINATION_CODE = 1234
+EXPECTED_TERMINATION_CODE = 123
+
+ANSI_CODE_TERMINATOR = re.compile("[@-~]")
 
 # other components may turn it on in order to avoid grouping output lines into one event
 io_animation_required = False
 
+_console_allocated = False
+
 
 class Runner:
     def __init__(self) -> None:
@@ -83,7 +91,14 @@ class Runner:
                 shutil.rmtree(os.path.join(THONNY_USER_DIR, item), True)
 
     def start(self) -> None:
-        self._check_alloc_console()
+        global _console_allocated
+        try:
+            self._check_alloc_console()
+            _console_allocated = True
+        except Exception:
+            logging.getLogger("thonny").exception("Problem allocating console")
+            _console_allocated = False
+
         self.restart_backend(False, True)
         # temporary
         self._remove_obsolete_jedi_copies()
@@ -156,11 +171,11 @@ class Runner:
         )
 
         get_workbench().add_command(
-            "softreboot",
+            "ctrld",
             "run",
             _("Send EOF / Soft reboot"),
-            self.soft_reboot,
-            self.soft_reboot_enabled,
+            self.ctrld,
+            self.ctrld_enabled,
             group=100,
             default_sequence="<Control-d>",
             extra_sequences=["<<CtrlDInText>>"],
@@ -231,6 +246,8 @@ class Runner:
         if "id" not in cmd:
             cmd["id"] = generate_command_id()
 
+        cmd["local_cwd"] = get_workbench().get_local_cwd()
+
         # Offer the command
         logging.debug("RUNNER Sending: %s, %s", cmd.name, cmd)
         response = self._proxy.send_command(cmd)
@@ -375,12 +392,18 @@ class Runner:
 
     def _cmd_interrupt(self) -> None:
         if self._proxy is not None:
-            self._proxy.interrupt()
+            if _console_allocated:
+                self._proxy.interrupt()
+            else:
+                messagebox.showerror(
+                    "No console",
+                    "Can't interrupt as console was not allocated.\n\nUse Stop/Restart instead.",
+                )
         else:
             logging.warning("Interrupting without proxy")
 
     def _cmd_interrupt_enabled(self) -> bool:
-        if not self._proxy or not self._proxy.is_functional():
+        if not self._proxy or not self._proxy.is_connected():
             return False
         # TODO: distinguish command and Ctrl+C shortcut
 
@@ -413,15 +436,25 @@ class Runner:
     def disconnect_enabled(self):
         return hasattr(self.get_backend_proxy(), "disconnect")
 
-    def soft_reboot(self):
+    def ctrld(self):
         proxy = self.get_backend_proxy()
-        if hasattr(proxy, "_soft_reboot_and_run_main"):
-            return proxy._soft_reboot_and_run_main()
-        return None
+        if not proxy:
+            return
 
-    def soft_reboot_enabled(self):
+        if get_shell().has_pending_input():
+            messagebox.showerror(
+                "Can't perform this action",
+                "Ctrl+D only has effect on an empty line / prompt.\n"
+                + "Submit current input (press ENTER) and try again",
+            )
+            return
+
+        proxy.send_command(EOFCommand())
+        self._set_state("running")
+
+    def ctrld_enabled(self):
         proxy = self.get_backend_proxy()
-        return proxy and proxy.is_functional() and hasattr(proxy, "_soft_reboot_and_run_main")
+        return proxy and proxy.is_connected()
 
     def _poll_vm_messages(self) -> None:
         """I chose polling instead of event_generate in listener thread,
@@ -462,7 +495,7 @@ class Runner:
                 "other messages don't affect the state"
 
             if "cwd" in msg:
-                if not self.has_own_filesystem():
+                if not self.supports_remote_files():
                     get_workbench().set_local_cwd(msg["cwd"])
 
             # Publish the event
@@ -495,7 +528,7 @@ class Runner:
         except Exception:
             logging.exception("Failed retrieving backend faults")
 
-        err = err.strip() + " Use 'Stop/Restart' to restart ...\n"
+        err = err.strip() + " Use 'Stop/Restart' to restart.\n"
 
         if returncode != EXPECTED_TERMINATION_CODE:
             get_workbench().event_generate("ProgramOutput", stream_name="stderr", data="\n" + err)
@@ -556,7 +589,7 @@ class Runner:
         return self._proxy
 
     def _check_alloc_console(self) -> None:
-        if sys.executable.endswith("thonny.exe") or sys.executable.endswith("pythonw.exe"):
+        if sys.executable.endswith("pythonw.exe"):
             # These don't have console allocated.
             # Console is required for sending interrupts.
 
@@ -566,9 +599,7 @@ class Runner:
 
             kernel32 = ctypes.WinDLL("kernel32", use_last_error=True)
 
-            exe = sys.executable.replace("thonny.exe", "python.exe").replace(
-                "pythonw.exe", "python.exe"
-            )
+            exe = sys.executable.replace("pythonw.exe", "python.exe")
 
             cmd = [exe, "-c", "print('Hi!'); input()"]
             child = subprocess.Popen(
@@ -592,8 +623,23 @@ class Runner:
                 # Without flush the console window becomes visible, but Thonny can be still used
                 logging.getLogger("thonny").exception("Problem with finalizing console allocation")
 
-    def can_do_file_operations(self):
-        return self._proxy and self._proxy.can_do_file_operations()
+    def ready_for_remote_file_operations(self, show_message=False):
+        if not self._proxy or not self.supports_remote_files():
+            return False
+
+        ready = self._proxy.ready_for_remote_file_operations()
+
+        if not ready and show_message:
+            if self._proxy.is_connected():
+                msg = "Device is not connected"
+            else:
+                msg = (
+                    "Device is busy -- can't perform this action now."
+                    + "\nPlease wait or cancel current work and try again!",
+                )
+            messagebox.showerror("Can't complete", msg)
+
+        return ready
 
     def get_supported_features(self) -> Set[str]:
         if self._proxy is None:
@@ -601,11 +647,17 @@ class Runner:
         else:
             return self._proxy.get_supported_features()
 
-    def has_own_filesystem(self):
+    def supports_remote_files(self):
         if self._proxy is None:
             return False
         else:
-            return self._proxy.has_own_filesystem()
+            return self._proxy.supports_remote_files()
+
+    def supports_remote_directories(self):
+        if self._proxy is None:
+            return False
+        else:
+            return self._proxy.supports_remote_directories()
 
     def get_node_label(self):
         if self._proxy is None:
@@ -666,8 +718,7 @@ class BackendProxy:
         """
         pass
 
-    def is_functional(self):
-        """Used in MicroPython proxies"""
+    def is_connected(self):
         return True
 
     def get_local_executable(self):
@@ -681,19 +732,21 @@ class BackendProxy:
         """Used as files caption if back-end has separate files"""
         return "Back-end"
 
-    def has_own_filesystem(self):
+    def supports_remote_files(self):
+        """Whether remote file browser should be presented with this back-end"""
         return False
 
     def uses_local_filesystem(self):
+        """Whether it runs code from local files"""
         return True
 
-    def supports_directories(self):
-        return True
+    def supports_remote_directories(self):
+        return False
 
     def supports_trash(self):
         return True
 
-    def can_do_file_operations(self):
+    def ready_for_remote_file_operations(self):
         return False
 
     def get_cwd(self):
@@ -737,6 +790,11 @@ class SubprocessProxy(BackendProxy):
         env["THONNY_USER_DIR"] = THONNY_USER_DIR
         env["THONNY_FRONTEND_SYS_PATH"] = repr(sys.path)
 
+        env["THONNY_LANGUAGE"] = get_workbench().get_option("general.language")
+        env["FRIENDLY_TRACEBACK_LEVEL"] = str(
+            get_workbench().get_option("assistance.friendly_traceback_level")
+        )
+
         if get_workbench().in_debug_mode():
             env["THONNY_DEBUG"] = "1"
         elif "THONNY_DEBUG" in env:
@@ -760,9 +818,14 @@ class SubprocessProxy(BackendProxy):
             creationflags = subprocess.CREATE_NEW_PROCESS_GROUP
 
         debug("Starting the backend: %s %s", cmd_line, get_workbench().get_local_cwd())
+
+        extra_params = {}
+        if sys.version_info >= (3, 6):
+            extra_params["encoding"] = "utf-8"
+
         self._proc = subprocess.Popen(
             cmd_line,
-            # bufsize=0,
+            bufsize=0,
             stdin=subprocess.PIPE,
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
@@ -770,6 +833,7 @@ class SubprocessProxy(BackendProxy):
             env=env,
             universal_newlines=True,
             creationflags=creationflags,
+            **extra_params
         )
 
         # setup asynchronous output listeners
@@ -800,8 +864,11 @@ class SubprocessProxy(BackendProxy):
     def send_program_input(self, data):
         self._send_msg(InputSubmission(data))
 
-    def _is_disconnected(self):
-        return self._proc is None or self._proc.poll() is not None
+    def process_is_alive(self):
+        return self._proc is not None and self._proc.poll() is None
+
+    def is_connected(self):
+        return self.process_is_alive()
 
     def get_sys_path(self):
         return self._sys_path
@@ -838,12 +905,13 @@ class SubprocessProxy(BackendProxy):
                 self.cwd = msg["cwd"]
             message_queue.append(msg)
 
-            while len(message_queue) > 100:
+            if len(message_queue) > 50:
                 # Probably backend runs an infinite/long print loop.
                 # Throttle message thougput in order to keep GUI thread responsive.
-                sleep(0.1)
+                while len(message_queue) > 0:
+                    sleep(0.1)
 
-        while not self._is_disconnected():
+        while self.process_is_alive():
             data = stdout.readline()
             # debug("... read some stdout data", repr(data))
             if data == "":
@@ -878,7 +946,7 @@ class SubprocessProxy(BackendProxy):
 
     def _listen_stderr(self, stderr):
         # stderr is used only for debugger debugging
-        while not self._is_disconnected():
+        while self.process_is_alive():
             data = stderr.readline()
             if data == "":
                 break
@@ -933,7 +1001,7 @@ class SubprocessProxy(BackendProxy):
 
     def fetch_next_message(self):
         if not self._response_queue or len(self._response_queue) == 0:
-            if self._is_disconnected():
+            if not self.process_is_alive():
                 raise BackendTerminatedError(self._proc.returncode if self._proc else None)
             else:
                 return None
@@ -945,20 +1013,31 @@ class SubprocessProxy(BackendProxy):
             # combine available small output messages to one single message,
             # in order to put less pressure on UI code
 
+            wait_time = 0.01
+            total_wait_time = 0
             while True:
                 if len(self._response_queue) == 0:
-                    return msg
+                    if _ends_with_incomplete_ansi_code(msg["data"]) and total_wait_time < 0.1:
+                        # Allow reader to send the remaining part
+                        sleep(wait_time)
+                        total_wait_time += wait_time
+                        continue
+                    else:
+                        return msg
                 else:
                     next_msg = self._response_queue.popleft()
                     if (
                         next_msg.event_type == "ProgramOutput"
                         and next_msg["stream_name"] == msg["stream_name"]
-                        and len(msg["data"]) + len(next_msg["data"]) <= OUTPUT_MERGE_THRESHOLD
-                        and ("\n" not in msg["data"] or not io_animation_required)
+                        and (
+                            len(msg["data"]) + len(next_msg["data"]) <= OUTPUT_MERGE_THRESHOLD
+                            and ("\n" not in msg["data"] or not io_animation_required)
+                            or _ends_with_incomplete_ansi_code(msg["data"])
+                        )
                     ):
                         msg["data"] += next_msg["data"]
                     else:
-                        # not same type of message, put it back
+                        # not to be sent in the same block, put it back
                         self._response_queue.appendleft(next_msg)
                         return msg
 
@@ -966,6 +1045,16 @@ class SubprocessProxy(BackendProxy):
             return msg
 
 
+def _ends_with_incomplete_ansi_code(data):
+    pos = data.rfind("\033")
+    if pos == -1:
+        return False
+
+    # note ANSI_CODE_TERMINATOR also includes [
+    params_and_terminator = data[pos + 2 :]
+    return not ANSI_CODE_TERMINATOR.search(params_and_terminator)
+
+
 class CPythonProxy(SubprocessProxy):
     "abstract class"
 
@@ -1018,7 +1107,12 @@ class CPythonProxy(SubprocessProxy):
 
     def _loop_gui_update(self, force=False):
         if force or get_runner().is_waiting_toplevel_command():
-            self.send_command(InlineCommand("process_gui_events"))
+            try:
+                self.send_command(InlineCommand("process_gui_events"))
+            except OSError:
+                # the backend process may have been closed already
+                # https://github.com/thonny/thonny/issues/966
+                logging.getLogger("thonny").exception("Could not send process_gui_events")
 
         self._gui_update_loop_id = get_workbench().after(50, self._loop_gui_update)
 
@@ -1086,7 +1180,7 @@ class PrivateVenvCPythonProxy(CPythonProxy):
         # This way all students will have similar configuration
         # independently of system Python (if Thonny is used with system Python)
 
-        # NB! Cant run venv.create directly, because in Windows
+        # NB! Cant run venv.create directly, because in Windows bundle
         # it tries to link venv to thonny.exe.
         # Need to run it via proper python
         args = ["-m", "venv"]
@@ -1135,7 +1229,7 @@ class PrivateVenvCPythonProxy(CPythonProxy):
 
 class SameAsFrontendCPythonProxy(CPythonProxy):
     def __init__(self, clean):
-        super().__init__(clean, get_frontend_python())
+        super().__init__(clean, get_interpreter_for_subprocess())
 
     def fetch_next_message(self):
         msg = super().fetch_next_message()
@@ -1157,7 +1251,7 @@ class CustomCPythonProxy(CPythonProxy):
             used_interpreters.append(executable)
         get_workbench().set_option("CustomInterpreter.used_paths", used_interpreters)
 
-        super().__init__(clean, executable)
+        super().__init__(clean, get_interpreter_for_subprocess(executable))
 
     def fetch_next_message(self):
         msg = super().fetch_next_message()
@@ -1227,7 +1321,10 @@ def create_frontend_python_process(
     args, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
 ):
     """Used for running helper commands (eg. for installing plug-ins on by the plug-ins)"""
-    python_exe = get_frontend_python().replace("pythonw.exe", "python.exe")
+    if _console_allocated:
+        python_exe = get_interpreter_for_subprocess().replace("pythonw.exe", "python.exe")
+    else:
+        python_exe = get_interpreter_for_subprocess().replace("python.exe", "pythonw.exe")
     env = get_environment_for_python_subprocess(python_exe)
     env["PYTHONIOENCODING"] = "utf-8"
     env["PYTHONUNBUFFERED"] = "1"
@@ -1277,10 +1374,6 @@ class BackendTerminatedError(Exception):
         self.returncode = returncode
 
 
-def get_frontend_python():
-    return sys.executable.replace("thonny.exe", "python.exe").replace("pythonw.exe", "python.exe")
-
-
 def is_venv_interpreter_of_current_interpreter(executable):
     for location in [".", ".."]:
         cfg_path = os.path.join(location, "pyvenv.cfg")
@@ -1475,3 +1568,20 @@ class BlockingDialog(CommonDialogEx):
                 "Cancel current operation?", "Do you really want to cancel this operation?"
             ):
                 self._send_interrupt()
+
+
+def get_frontend_python():
+    # TODO: deprecated (name can be misleading)
+    warnings.warn("get_frontend_python is deprecated")
+    return get_interpreter_for_subprocess(sys.executable)
+
+
+def get_interpreter_for_subprocess(candidate=None):
+    if candidate is None:
+        candidate = sys.executable
+
+    pythonw = candidate.replace("python.exe", "pythonw.exe")
+    if not _console_allocated and os.path.exists(pythonw):
+        return pythonw
+    else:
+        return candidate.replace("pythonw.exe", "python.exe")
