@@ -21,6 +21,7 @@ from thonny.ui_utils import (
     show_dialog,
     lookup_style_option,
     CommonDialog,
+    tr_btn,
 )
 import tkinter as tk
 import webbrowser
@@ -32,7 +33,7 @@ _CLEAR_SHELL_DEFAULT_SEQ = select_sequence("<Control-l>", "<Command-k>")
 OUTPUT_SPLIT_REGEX = re.compile(r"(\x1B\[[0-?]*[ -/]*[@-~]|[\a\b\r])")
 NUMBER_SPLIT_REGEX = re.compile(r"((?<!\w)[-+]?[0-9]*\.?[0-9]+\b)")
 SIMPLE_URL_SPLIT_REGEX = re.compile(
-    r"(https?:\/\/[\w\/.:\-\?#=%]+[\w\/]|data:image\/[a-z]+;base64,[A-Za-z0-9\/=]+)"
+    r"(https?:\/\/[\w\/.:\-\?#=%]+[\w\/]|data:image\/[a-z]+;base64,[A-Za-z0-9\/=\+]+)"
 )
 
 INT_REGEX = re.compile(r"\d+")
@@ -199,6 +200,9 @@ class ShellView(tk.PanedWindow):
     def clear_shell(self):
         self.text._clear_shell()
 
+    def has_pending_input(self):
+        return self.text.has_pending_input()
+
     def report_exception(self, prelude=None, conclusion=None):
         if prelude is not None:
             self.text.direct_insert("end", prelude + "\n", ("stderr",))
@@ -230,6 +234,9 @@ class ShellView(tk.PanedWindow):
         if self.plotter is not None and self.plotter.winfo_ismapped():
             self.plotter.update_plot()
 
+    def update_tabs(self):
+        self.text.update_tabs()
+
     def resize_plotter(self):
         if len(self.panes()) > 1 and self.text.winfo_width() > 5:
             get_workbench().set_option("view.shell_sash_position", self.sash_coord(0)[0])
@@ -269,6 +276,7 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
 
     def __init__(self, master, view=None, cnf={}, **kw):
         self.view = view
+        kw["tabstyle"] = "wordprocessor"
         super().__init__(master, cnf, **kw)
 
         self._command_history = (
@@ -304,6 +312,7 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
         x_padding = 4
         io_vert_spacing = 10
         io_indent = 16 + x_padding
+        self.io_indent = io_indent
         code_indent = prompt_font.measure(">>> ") + x_padding
 
         self.tag_configure("command", lmargin1=code_indent, lmargin2=code_indent)
@@ -343,6 +352,8 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
 
         self.active_object_tags = set()
 
+        self.update_tabs()
+
         self.tag_raise("io_hyperlink")
         self.tag_raise("underline")
         self.tag_raise("strikethrough")
@@ -520,7 +531,11 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
             if stream_name == "stdout" and self.tty_mode:
                 tags |= self._get_ansi_tags()
 
-            if len(data) > self._get_squeeze_threshold() and "\n" not in data:
+            non_url_length = len(data)
+            for url_match in SIMPLE_URL_SPLIT_REGEX.finditer(data):
+                non_url_length -= url_match.end() - url_match.start()
+
+            if non_url_length > self._get_squeeze_threshold() and "\n" not in data:
                 self._io_cursor_offset = 0  # ignore the effect of preceding \r and \b
                 button_text = data[:40] + " …"
                 btn = tk.Label(
@@ -802,6 +817,18 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
         if focused_view is not None:
             focused_view.focus()
 
+    def update_tabs(self):
+        tab_chars = 8
+        tab_pixels = tk.font.nametofont("IOFont").measure("n" * tab_chars)
+
+        offset = self.io_indent
+        tabs = [offset]
+        for _ in range(20):
+            offset += tab_pixels
+            tabs.append(offset)
+
+        self.tag_configure("io", tabs=tabs, tabstyle="wordprocessor")
+
     def restart(self):
         self._insert_text_directly(
             # "\n============================== RESTART ==============================\n",
@@ -946,17 +973,24 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
                     _insert(parts[0], tags)
                     _insert(parts[1], tags + ("io_hyperlink",))
                     _insert(parts[2], tags)
-                    # self.tag_raise("io_hyperlink", "io")
-                    # self.tag_raise("io_hyperlink", "stderr")
-                    # self.tag_raise("io_hyperlink", "stdout")
                 else:
-                    _insert_and_highlight_urls(line, tags)
+                    parts = re.split(r"(\'[^\']+\.pyw?\')", line, flags=re.IGNORECASE)
+                    if len(parts) == 3 and os.path.exists(os.path.expanduser(parts[1][1:-1])):
+                        match = re.search(r"\S", line)
+                        _insert(line[: match.start()], tags)
+                        _insert(line[match.start() :], tags + ("io_hyperlink",))
+                    else:
+                        _insert_and_highlight_urls(line, tags)
         else:
             _insert_and_highlight_urls(txt, tags)
 
         # self._print_marks("after output")
         # output_insert mark will move automatically because of its gravity
 
+    def has_pending_input(self):
+        pending = self.get("input_start", "end-1c")
+        return bool(pending)
+
     def _try_submit_input(self):
         # see if there is already enough inputted text to submit
         input_text = self.get("input_start", "insert")
@@ -1001,8 +1035,7 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
             self._submit_input(submittable_text)
 
     def _editing_allowed(self):
-        # TODO: get rid of this
-        return True
+        return get_runner() is not None
 
     def _extract_submittable_input(self, input_text, tail):
 
@@ -1252,10 +1285,19 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
     def _handle_hyperlink(self, event):
         try:
             line = self.get("insert linestart", "insert lineend")
-            matches = re.findall(r'File "([^"]+)", line (\d+)', line)
-            if len(matches) == 1 and len(matches[0]) == 2:
-                filename, lineno = matches[0]
-                lineno = int(lineno)
+            # Python stacktrace
+            matches = list(re.finditer(r'File "(?P<file>[^"]+)", line (?P<line>\d+)', line))
+            if not matches:
+                # Friendly traceback
+                matches = list(
+                    re.finditer(
+                        r"\b(?P<line>\d+)\b.+'(?P<file>[^\']+\.pyw?)'", line, flags=re.IGNORECASE
+                    )
+                )
+
+            if len(matches) == 1:
+                filename = os.path.expanduser(matches[0].group("file"))
+                lineno = int(matches[0].group("line"))
                 if os.path.exists(filename) and os.path.isfile(filename):
                     # TODO: better use events instead direct referencing
                     get_workbench().get_editor_notebook().show_file(
@@ -1272,6 +1314,10 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
             traceback.print_exc()
 
     def _show_user_exception(self, user_exception):
+        fr_tr = user_exception.get("friendly_traceback", None)
+        if fr_tr is not None:
+            self._format_friendly_traceback(fr_tr)
+            return
 
         for line, frame_id, *_ in user_exception["items"]:
 
@@ -1289,6 +1335,13 @@ class BaseShellText(EnhancedTextWithLogging, PythonText):
 
             self._insert_text_directly(line, tags)
 
+    def _format_friendly_traceback(self, s):
+        tags = ("io", "stderr")
+
+        lines = s.splitlines(True)
+        for line in lines:
+            self._insert_text_directly(line, tags)
+
     def _discard_old_content(self):
         max_lines = max(get_workbench().get_option("shell.max_lines"), 0)
         proposed_cut = self.index("end -%d lines linestart" % max_lines)
@@ -1422,19 +1475,19 @@ class SqueezedTextDialog(CommonDialog):
         button_frame.grid(row=3, column=0, padx=padding, pady=padding, sticky="nswe")
         button_frame.columnconfigure(2, weight=1)
 
-        copy_caption = _("Copy to clipboard")
+        copy_caption = tr_btn("Copy to clipboard")
         copy_button = ttk.Button(
             button_frame, text=copy_caption, width=len(copy_caption), command=self._on_copy
         )
         copy_button.grid(row=0, column=1, sticky="w", padx=(0, padding))
 
-        expand_caption = _("Expand in Shell")
+        expand_caption = tr_btn("Expand in Shell")
         expand_button = ttk.Button(
             button_frame, text=expand_caption, width=len(expand_caption), command=self._on_expand
         )
         expand_button.grid(row=0, column=2, sticky="e", padx=padding)
 
-        close_button = ttk.Button(button_frame, text="Close", command=self._on_close)
+        close_button = ttk.Button(button_frame, text=tr_btn("Close"), command=self._on_close)
         close_button.grid(row=0, column=3, sticky="e")
 
         self.bind("<Escape>", self._on_close, True)
