@@ -4,10 +4,15 @@ import shlex
 import shutil
 import subprocess
 
+
 def run_in_terminal(cmd, cwd, env_overrides={}, keep_open=True, title=None):
     from thonny.running import get_environment_with_overrides
+
     env = get_environment_with_overrides(env_overrides)
-            
+
+    if not cwd or not os.path.exists(cwd):
+        cwd = os.getcwd()
+
     if platform.system() == "Windows":
         _run_in_terminal_in_windows(cmd, cwd, env, keep_open, title)
     elif platform.system() == "Linux":
@@ -17,10 +22,12 @@ def run_in_terminal(cmd, cwd, env_overrides={}, keep_open=True, title=None):
     else:
         raise RuntimeError("Can't launch terminal in " + platform.system())
 
+
 def open_system_shell(cwd, env_overrides={}):
     from thonny.running import get_environment_with_overrides
+
     env = get_environment_with_overrides(env_overrides)
-    
+
     if platform.system() == "Darwin":
         _run_in_terminal_in_macos([], cwd, env_overrides, True)
     elif platform.system() == "Windows":
@@ -32,13 +39,16 @@ def open_system_shell(cwd, env_overrides={}):
     else:
         raise RuntimeError("Can't launch terminal in " + platform.system())
 
+
 def _add_to_path(directory, path):
     # Always prepending to path may seem better, but this could mess up other things.
     # If the directory contains only one Python distribution executables, then
     # it probably won't be in path yet and therefore will be prepended.
-    if (directory in path.split(os.pathsep)
+    if (
+        directory in path.split(os.pathsep)
         or platform.system() == "Windows"
-        and directory.lower() in path.lower().split(os.pathsep)):
+        and directory.lower() in path.lower().split(os.pathsep)
+    ):
         return path
     else:
         return directory + os.pathsep + path
@@ -48,34 +58,31 @@ def _run_in_terminal_in_windows(cmd, cwd, env, keep_open, title=None):
     if keep_open:
         # Yes, the /K argument has weird quoting. Can't explain this, but it works
         quoted_args = " ".join(map(lambda s: s if s == "&" else '"' + s + '"', cmd))
-        cmd_line = ("""start {title} /D "{cwd}" /W cmd /K "{quoted_args}" """
-                    .format(cwd=cwd,
-                            quoted_args=quoted_args,
-                            title='"' + title + '"' if title else ""))
-    
+        cmd_line = """start {title} /D "{cwd}" /W cmd /K "{quoted_args}" """.format(
+            cwd=cwd, quoted_args=quoted_args, title='"' + title + '"' if title else ""
+        )
+
         subprocess.Popen(cmd_line, cwd=cwd, env=env, shell=True)
     else:
-        subprocess.Popen(cmd, creationflags=subprocess.CREATE_NEW_CONSOLE,
-                         cwd=cwd, env=env)
+        subprocess.Popen(cmd, creationflags=subprocess.CREATE_NEW_CONSOLE, cwd=cwd, env=env)
 
 
 def _run_in_terminal_in_linux(cmd, cwd, env, keep_open):
     def _shellquote(s):
         return subprocess.list2cmdline([s])
-    
+
     term_cmd = _get_linux_terminal_command()
-    
-    
+
     if isinstance(cmd, list):
         cmd = " ".join(map(_shellquote, cmd))
-        
+
     if keep_open:
         # http://stackoverflow.com/a/4466566/261181
         core_cmd = "{cmd}; exec bash -i".format(cmd=cmd)
         in_term_cmd = "bash -c {core_cmd}".format(core_cmd=_shellquote(core_cmd))
     else:
         in_term_cmd = cmd
-    
+
     if term_cmd == "lxterminal":
         # https://www.raspberrypi.org/forums/viewtopic.php?t=221490
         whole_cmd = "{term_cmd} --command={in_term_cmd}".format(
@@ -85,7 +92,6 @@ def _run_in_terminal_in_linux(cmd, cwd, env, keep_open):
         whole_cmd = "{term_cmd} -e {in_term_cmd}".format(
             term_cmd=term_cmd, in_term_cmd=_shellquote(in_term_cmd)
         )
-        
 
     subprocess.Popen(whole_cmd, cwd=cwd, env=env, shell=True)
 
@@ -101,32 +107,27 @@ def _run_in_terminal_in_macos(cmd, cwd, env_overrides, keep_open):
         if env_overrides[key] is None:
             cmds += "; unset " + key
         else:
-            cmds += "; export {key}={value}".format(key=key, 
-                                                    value=_shellquote(env_overrides[key]))
-    
+            cmds += "; export {key}={value}".format(key=key, value=_shellquote(env_overrides[key]))
+
     if cmd:
         if isinstance(cmd, list):
             cmd = " ".join(map(_shellquote, cmd))
         cmds += "; " + cmd
-    
+
     if not keep_open:
         cmds += "; exit"
 
     # The script will be sent to Terminal with 'do script' command, which takes a string.
     # We'll prepare an AppleScript string literal for this
     # (http://stackoverflow.com/questions/10667800/using-quotes-in-a-applescript-string):
-    cmd_as_apple_script_string_literal = (
-        '"' + cmds.replace("\\", "\\\\").replace('"', '\\"') + '"'
-    )
+    cmd_as_apple_script_string_literal = '"' + cmds.replace("\\", "\\\\").replace('"', '\\"') + '"'
 
     # When Terminal is not open, then do script opens two windows.
     # do script ... in window 1 would solve this, but if Terminal is already
     # open, this could run the script in existing terminal (in undesirable env on situation)
     # That's why I need to prepare two variations of the 'do script' command
     doScriptCmd1 = """        do script %s """ % cmd_as_apple_script_string_literal
-    doScriptCmd2 = (
-        """        do script %s in window 1 """ % cmd_as_apple_script_string_literal
-    )
+    doScriptCmd2 = """        do script %s in window 1 """ % cmd_as_apple_script_string_literal
 
     # The whole AppleScript will be executed with osascript by giving script
     # lines as arguments. The lines containing our script need to be shell-quoted:
@@ -153,19 +154,21 @@ def _run_in_terminal_in_macos(cmd, cwd, env_overrides, keep_open):
 
     subprocess.Popen(cmd_line, cwd=cwd, shell=True)
 
+
 def _get_linux_terminal_command():
     xte = shutil.which("x-terminal-emulator")
     if xte:
-        if (os.path.realpath(xte).endswith("/lxterminal")
-            and shutil.which("lxterminal")):
+        if os.path.realpath(xte).endswith("/lxterminal") and shutil.which("lxterminal"):
             # need to know exact program, because it needs special treatment
             return "lxterminal"
         else:
             return "x-terminal-emulator"
     # Older konsole didn't pass on the environment
     elif shutil.which("konsole"):
-        if (shutil.which("gnome-terminal")
-            and "gnome" in os.environ.get("DESKTOP_SESSION", "").lower()):
+        if (
+            shutil.which("gnome-terminal")
+            and "gnome" in os.environ.get("DESKTOP_SESSION", "").lower()
+        ):
             return "gnome-terminal"
         else:
             return "konsole"
@@ -179,4 +182,3 @@ def _get_linux_terminal_command():
         return "xterm"
     else:
         raise RuntimeError("Don't know how to open terminal emulator")
-    
