@@ -16,10 +16,7 @@ def joo():
 
 def test_regular_closed():
 
-    expected_local = {('5.4', '5.12'),
-                      ('6.10', '6.18'),
-                      ('6.21', '6.29'),
-                      }
+    expected_local = {("5.4", "5.12"), ("6.10", "6.18"), ("6.21", "6.29")}
 
     text_widget = tkinter.Text()
     text_widget.insert("end", TEST_STR1)
@@ -30,12 +27,3 @@ def test_regular_closed():
 
     assert actual_local == expected_local
     print("Passed.")
-
-
-def run_tests():
-    test_regular_closed()
-
-if __name__ == "__main__":
-    print("Test input: ")
-    print(TEST_STR1)
-    run_tests()
\ No newline at end of file
