@@ -28,6 +28,7 @@ from thonny.tktextext import TweakableText
 import sys
 from _tkinter import TclError
 import webbrowser
+from thonny.languages import get_button_padding
 
 PARENS_REGEX = re.compile(r"[\(\)\{\}\[\]]")
 
@@ -39,7 +40,6 @@ class CommonDialog(tk.Toplevel):
 
     def _unlock_on_focus_in(self, event):
         if not self.winfo_ismapped():
-            print("unlocking")
             focussed_widget = self.focus_get()
             self.deiconify()
             if focussed_widget:
@@ -1020,16 +1020,24 @@ class ToolTip:
         x = x + self.widget.winfo_rootx() + 27
         y = y + cy + self.widget.winfo_rooty() + self.widget.winfo_height() + 2
         self.tipwindow = tw = tk.Toplevel(self.widget)
-        tw.wm_overrideredirect(1)
+        if running_on_mac_os():
+            try:
+                # Must be the first thing to do after creating window
+                # https://wiki.tcl-lang.org/page/MacWindowStyle
+                tw.tk.call(
+                    "::tk::unsupported::MacWindowStyle", "style", tw._w, "help", "noActivates"
+                )
+            except tk.TclError:
+                pass
+        else:
+            tw.wm_overrideredirect(1)
+
+        tw.wm_geometry("+%d+%d" % (x, y))
+
         if running_on_mac_os():
             # TODO: maybe it's because of Tk 8.5, not because of Mac
             tw.wm_transient(self.widget)
-        tw.wm_geometry("+%d+%d" % (x, y))
-        try:
-            # For Mac OS
-            tw.tk.call("::tk::unsupported::MacWindowStyle", "style", tw._w, "help", "noActivates")
-        except tk.TclError:
-            pass
+
         label = tk.Label(tw, text=self.text, **self.options)
         label.pack()
         # get_workbench().bind("WindowFocusOut", self.hidetip, True)
@@ -1390,7 +1398,7 @@ def center_window(win, master=None):
     return assign_geometry(win, master)
 
 
-def assign_geometry(win, master=None):
+def assign_geometry(win, master=None, min_left=0, min_top=0):
 
     if master is None:
         master = tk._default_root
@@ -1426,6 +1434,9 @@ def assign_geometry(win, master=None):
     left = master.winfo_rootx() + master.winfo_width() // 2 - width // 2
     top = master.winfo_rooty() + master.winfo_height() // 2 - height // 2
 
+    left = max(left, min_left)
+    top = max(top, min_top)
+
     if saved_size:
         win.geometry("%dx%d+%d+%d" % (width, height, left, top))
     else:
@@ -1724,6 +1735,7 @@ class SubprocessDialog(CommonDialog):
         )  # escape-close only if process has completed
         self.protocol("WM_DELETE_WINDOW", self._close)
 
+        self.update_idletasks()
         self._start_listening()
 
     def _start_listening(self):
@@ -1762,11 +1774,18 @@ class SubprocessDialog(CommonDialog):
                 self.button["text"] = _("OK")
                 self.button.focus_set()
                 if self.returncode != 0:
-                    self.text.direct_insert("end", "\n\nReturn code: ", ("stderr",))
+                    self.text.direct_insert(
+                        "end",
+                        "\n\nProcess failed, return code: %s\n" % self.returncode,
+                        ("stderr",),
+                    )
+                    self.update_idletasks()
+                    self.text.see("end")
                 elif self._autoclose:
                     self._close()
                 else:
                     self.text.direct_insert("end", "\n\n" + self._conclusion)
+                    self.update_idletasks()
                     self.text.see("end")
 
         poll_output_events()
@@ -2058,7 +2077,10 @@ def handle_mistreated_latin_shortcuts(registry, event):
                     handler()
 
 
-def show_dialog(dlg, master=None, geometry=True):
+def show_dialog(dlg, master=None, geometry=True, min_left=0, min_top=0):
+    if getattr(dlg, "closed", False):
+        return
+
     if master is None:
         master = tk._default_root
 
@@ -2072,7 +2094,7 @@ def show_dialog(dlg, master=None, geometry=True):
         if isinstance(geometry, str):
             dlg.geometry(geometry)
         else:
-            assign_geometry(dlg, master)
+            assign_geometry(dlg, master, min_left, min_top)
         # dlg.wm_deiconify()
 
     try:
@@ -2088,6 +2110,8 @@ def show_dialog(dlg, master=None, geometry=True):
     master.winfo_toplevel().lift()
     master.winfo_toplevel().focus_force()
     master.winfo_toplevel().grab_set()
+    if running_on_mac_os():
+        master.winfo_toplevel().grab_release()
 
     if focused_widget is not None:
         try:
@@ -2262,6 +2286,18 @@ def ems_to_pixels(x):
     return int(EM_WIDTH * x)
 
 
+_btn_padding = None
+
+
+def tr_btn(s):
+    """Translates button caption, adds padding to make sure text fits"""
+    global _btn_padding
+    if _btn_padding is None:
+        _btn_padding = get_button_padding()
+
+    return _btn_padding + _(s) + _btn_padding
+
+
 if __name__ == "__main__":
     root = tk.Tk()
     closa = ClosableNotebook(root)
