@@ -89,6 +89,7 @@ OBSOLETE_PLUGINS = [
     "thonnycontrib.micropython",
     "thonnycontrib.circuitpython",
     "thonnycontrib.microbit",
+    "thonnycontrib.esp",
 ]
 
 
@@ -244,8 +245,12 @@ class Workbench(tk.Tk):
         language_code = self.get_option("general.language")
         if language_code in languages.LANGUAGES_DICT:
             path = os.path.join(os.path.dirname(__file__), "locale")
-            language = gettext.translation("thonny", path, [language_code])
-            language.install()
+            try:
+                language = gettext.translation("thonny", path, [language_code])
+                language.install()
+            except Exception:
+                traceback.print_exc()
+                self.report_exception("Can't load language " + language_code)
 
     def _get_logging_level(self) -> int:
         if self.in_debug_mode():
@@ -506,16 +511,16 @@ class Workbench(tk.Tk):
         self.set_default("CustomInterpreter.used_paths", [])
         self.set_default("CustomInterpreter.path", "")
 
+        from thonny import running_config_page
+
         self.add_backend(
             "SameAsFrontend",
             running.SameAsFrontendCPythonProxy,
             _("The same interpreter which runs Thonny (default)"),
-            running.get_frontend_python(),
+            running_config_page.SameAsFrontEndConfigurationPage,
             "1",
         )
 
-        from thonny import running_config_page
-
         self.add_backend(
             "CustomCPython",
             running.CustomCPythonProxy,
@@ -528,9 +533,7 @@ class Workbench(tk.Tk):
             "PrivateVenv",
             running.PrivateVenvCPythonProxy,
             _("A special virtual environment (deprecated)"),
-            _("This virtual environment is automatically maintained by Thonny.\n")
-            + _("Location: ")
-            + running.get_private_venv_path(),
+            running_config_page.PrivateVenvConfigurationPage,
             "z",
         )
 
@@ -936,8 +939,10 @@ class Workbench(tk.Tk):
 
         if image:
             _image = self.get_image(image)  # type: Optional[tk.PhotoImage]
+            _disabled_image = self.get_image(image, disabled=True)
         else:
             _image = None
+            _disabled_image = None
 
         if not accelerator and sequence:
             accelerator = sequence_to_accelerator(sequence)
@@ -994,6 +999,7 @@ class Workbench(tk.Tk):
             self._add_toolbar_button(
                 command_id,
                 _image,
+                _disabled_image,
                 command_label,
                 caption,
                 caption if alternative_caption is None else alternative_caption,
@@ -1085,9 +1091,8 @@ class Workbench(tk.Tk):
 
         # assing names to related classes
         proxy_class.backend_name = name  # type: ignore
-        if not isinstance(config_page_constructor, str):
-            if not getattr(config_page_constructor, "backend_name", None):
-                config_page_constructor.backend_name = name
+        if not getattr(config_page_constructor, "backend_name", None):
+            config_page_constructor.backend_name = name
 
     def add_ui_theme(
         self,
@@ -1405,7 +1410,12 @@ class Workbench(tk.Tk):
             label: translated label, used only when menu with given name doesn't exist yet
         """
         if name not in self._menus:
-            menu = tk.Menu(self._menubar, **get_style_configuration("Menu"))
+            if running_on_mac_os():
+                conf = {}
+            else:
+                conf = get_style_configuration("Menu")
+
+            menu = tk.Menu(self._menubar, **conf)
             menu["postcommand"] = lambda: self._update_menu(menu, name)
             self._menubar.add_cascade(label=label if label else name, menu=menu)
 
@@ -1451,7 +1461,9 @@ class Workbench(tk.Tk):
         """Returns thonny package directory"""
         return os.path.dirname(sys.modules["thonny"].__file__)
 
-    def get_image(self, filename: str, tk_name: Optional[str] = None) -> tk.PhotoImage:
+    def get_image(
+        self, filename: str, tk_name: Optional[str] = None, disabled=False
+    ) -> tk.PhotoImage:
 
         if filename in self._image_mapping_by_theme[self._current_theme_name]:
             filename = self._image_mapping_by_theme[self._current_theme_name][filename]
@@ -1468,6 +1480,13 @@ class Workbench(tk.Tk):
                 elif os.path.exists(filename + ".gif"):
                     filename = filename + ".gif"
 
+        if disabled:
+            filename = os.path.join(
+                os.path.dirname(filename), "_disabled_" + os.path.basename(filename)
+            )
+            if not os.path.exists(filename):
+                return None
+
         # are there platform-specific variants?
         plat_filename = filename[:-4] + "_" + platform.system() + ".png"
         if os.path.exists(plat_filename):
@@ -1707,6 +1726,14 @@ class Workbench(tk.Tk):
         ]:
             tk_font.nametofont(io_name).configure(family=io_font_family, size=io_font_size)
 
+        try:
+            shell = self.get_view("ShellView", create=False)
+        except Exception:
+            # shell may be not created yet
+            pass
+        else:
+            shell.update_tabs()
+
         tk_font.nametofont("EditorFont").configure(family=editor_font_family, size=editor_font_size)
         tk_font.nametofont("SmallEditorFont").configure(
             family=editor_font_family, size=editor_font_size - 2
@@ -1735,15 +1762,17 @@ class Workbench(tk.Tk):
                 size=round(editor_font_size * small_size_factor)
             )
 
-        style = ttk.Style()
+        # Update Treeview font and row height
         if running_on_mac_os():
             treeview_font_size = int(editor_font_size * 0.7 + 4)
-            rowheight = int(treeview_font_size * 1.2 + self.scale(4))
         else:
             treeview_font_size = int(editor_font_size * 0.7 + 2)
-            rowheight = int(treeview_font_size * 2.0 + self.scale(3))
 
-        tk_font.nametofont("TreeviewFont").configure(size=treeview_font_size)
+        treeview_font = tk_font.nametofont("TreeviewFont")
+        treeview_font.configure(size=treeview_font_size)
+        rowheight = round(treeview_font.metrics("linespace") * 1.2)
+
+        style = ttk.Style()
         style.configure("Treeview", rowheight=rowheight)
 
         if self._editor_notebook is not None:
@@ -1760,6 +1789,7 @@ class Workbench(tk.Tk):
         self,
         command_id: str,
         image: Optional[tk.PhotoImage],
+        disabled_image: Optional[tk.PhotoImage],
         command_label: str,
         caption: str,
         alternative_caption: str,
@@ -1794,10 +1824,15 @@ class Workbench(tk.Tk):
         else:
             button_width = None
 
+        if disabled_image is not None:
+            image_spec = [image, "disabled", disabled_image]
+        else:
+            image_spec = image
+
         button = ttk.Button(
             group_frame,
             command=handler,
-            image=image,
+            image=image_spec,
             style="Toolbutton",
             state=tk.NORMAL,
             text=caption,
